@@ -1,6 +1,6 @@
 /* $Xorg: button.c,v 1.3 2000/08/17 19:55:08 cpqbld Exp $ */
 /*
- * Copyright 1999-2001,2002 by Thomas E. Dickey
+ * Copyright 1999-2002,2003 by Thomas E. Dickey
  *
  *                         All Rights Reserved
  *
@@ -50,7 +50,7 @@
  * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
  * SOFTWARE.
  */
-/* $XFree86: xc/programs/xterm/button.c,v 3.67 2002/04/28 19:04:19 dickey Exp $ */
+/* $XFree86: xc/programs/xterm/button.c,v 3.73 2003/03/23 02:01:38 dickey Exp $ */
 
 /*
 button.c	Handles button events in the terminal emulator.
@@ -140,7 +140,17 @@ static int saveStartWRow, saveStartWCol;
 /* Multi-click handling */
 static int numberOfClicks = 0;
 static Time lastButtonUpTime = 0;
+
+#if OPT_READLINE
+static Time lastButtonDownTime = 0;
+static int ExtendingSelection = 0;
+static Time lastButton3UpTime = 0;
+static Time lastButton3DoubleDownTime = 0;
+static int lastButton3row, lastButton3col;	/* At the release time */
+#endif /* OPT_READLINE */
+
 typedef int SelectUnit;
+
 #define SELECTCHAR 0
 #define SELECTWORD 1
 #define SELECTLINE 2
@@ -168,6 +178,8 @@ static void SelectionReceived PROTO_XT_SEL_CB_ARGS;
 static void StartSelect(int startrow, int startcol);
 static void TrackDown(XButtonEvent * event);
 static void _OwnSelection(XtermWidget termw, String * selections, Cardinal count);
+static void do_select_end(Widget w, XEvent * event, String * params,
+			  Cardinal * num_params, Bool use_cursor_loc);
 
 Boolean
 SendMousePosition(Widget w, XEvent * event)
@@ -306,9 +318,9 @@ SendLocatorPosition(Widget w, XEvent * event)
     LocatorCoords(row, col, event->xbutton.x, event->xbutton.y, oor);
 
     /*
-       * DECterm mouse:
-       *
-       * ESCAPE '[' event ; mask ; row ; column '&' 'w'
+     * DECterm mouse:
+     *
+     * ESCAPE '[' event ; mask ; row ; column '&' 'w'
      */
     reply.a_type = CSI;
 
@@ -634,6 +646,182 @@ CheckLocatorPosition(Widget w, XEvent * event)
 }
 #endif /* OPT_DEC_LOCATOR */
 
+#if OPT_READLINE
+static int
+isClick1_clean(XEvent * event)
+{
+    TScreen *screen = &term->screen;
+    int delta;
+
+    if (!(event->type == ButtonPress || event->type == ButtonRelease)
+    /* Disable on Shift-Click-1, including the application-mouse modes */
+	|| (KeyModifiers & ShiftMask)
+	|| (screen->send_mouse_pos != MOUSE_OFF)	/* Kinda duplicate... */
+	||ExtendingSelection)	/* Was moved */
+	return 0;
+    if (event->type != ButtonRelease)
+	return 0;
+    if (lastButtonDownTime == (Time) 0)		/* first time or once in a blue moon */
+	delta = term->screen.multiClickTime + 1;
+    else if (event->xbutton.time > lastButtonDownTime)	/* most of the time */
+	delta = event->xbutton.time - lastButtonDownTime;
+    else			/* time has rolled over since lastButtonUpTime */
+	delta = (((Time) ~ 0) - lastButtonDownTime) + event->xbutton.time;
+    return delta <= term->screen.multiClickTime;
+}
+
+static int
+isDoubleClick3(XEvent * event)
+{
+    int delta;
+
+    if (event->type != ButtonRelease
+	|| (KeyModifiers & ShiftMask)
+	|| event->xbutton.button != Button3) {
+	lastButton3UpTime = 0;	/* Disable the cached info */
+	return 0;
+    }
+    /* Process Btn3Release. */
+    if (lastButton3DoubleDownTime == (Time) 0)	/* No previous click
+						   or once in a blue moon */
+	delta = term->screen.multiClickTime + 1;
+    else if (event->xbutton.time > lastButton3DoubleDownTime)	/* most of the time */
+	delta = event->xbutton.time - lastButton3DoubleDownTime;
+    else			/* time has rolled over since lastButton3DoubleDownTime */
+	delta = (((Time) ~ 0) - lastButton3DoubleDownTime) + event->xbutton.time;
+    if (delta <= term->screen.multiClickTime) {
+	/* Double click */
+	int row, col;
+
+	/* Cannot check ExtendingSelection, since mouse-3 always sets it */
+	PointToRowCol(event->xbutton.y, event->xbutton.x, &row, &col);
+	if (row == lastButton3row && col == lastButton3col) {
+	    lastButton3DoubleDownTime = 0;	/* Disable the third click */
+	    return 1;
+	}
+    }
+    /* Not a double click, memorize for future check. */
+    lastButton3UpTime = event->xbutton.time;
+    PointToRowCol(event->xbutton.y, event->xbutton.x,
+		  &lastButton3row, &lastButton3col);
+    return 0;
+}
+
+static int
+CheckSecondPress3(XEvent * event)
+{
+    int delta, row, col;
+
+    if (event->type != ButtonPress
+	|| (KeyModifiers & ShiftMask)
+	|| event->xbutton.button != Button3) {
+	lastButton3DoubleDownTime = 0;	/* Disable the cached info */
+	return 0;
+    }
+    /* Process Btn3Press. */
+    if (lastButton3UpTime == (Time) 0)	/* No previous click
+					   or once in a blue moon */
+	delta = term->screen.multiClickTime + 1;
+    else if (event->xbutton.time > lastButton3UpTime)	/* most of the time */
+	delta = event->xbutton.time - lastButton3UpTime;
+    else			/* time has rolled over since lastButton3UpTime */
+	delta = (((Time) ~ 0) - lastButton3UpTime) + event->xbutton.time;
+    if (delta <= term->screen.multiClickTime) {
+	PointToRowCol(event->xbutton.y, event->xbutton.x, &row, &col);
+	if (row == lastButton3row && col == lastButton3col) {
+	    /* A candidate for a double-click */
+	    lastButton3DoubleDownTime = event->xbutton.time;
+	    PointToRowCol(event->xbutton.y, event->xbutton.x,
+			  &lastButton3row, &lastButton3col);
+	    return 1;
+	}
+	lastButton3UpTime = 0;	/* Disable the info about the previous click */
+    }
+    /* Either too long, or moved, disable. */
+    lastButton3DoubleDownTime = 0;
+    return 0;
+}
+
+static int
+rowOnCurrentLine(int line, int *deltap)		/* must be XButtonEvent */
+{
+    TScreen *screen = &term->screen;
+    int l1, l2;
+
+    *deltap = 0;
+    if (line == screen->cur_row)
+	return 1;
+
+    if (line < screen->cur_row)
+	l1 = line, l2 = screen->cur_row;
+    else
+	l2 = line, l1 = screen->cur_row;
+    l1--;
+    while (++l1 < l2)
+	if (!ScrnTstWrapped(screen, l1))
+	    return 0;
+    /* Everything is on one "wrapped line" now */
+    *deltap = line - screen->cur_row;
+    return 1;
+}
+
+static int
+eventRow(XEvent * event)	/* must be XButtonEvent */
+{
+    TScreen *screen = &term->screen;
+
+    return (event->xbutton.y - screen->border) / FontHeight(screen);
+}
+
+static int
+eventColBetween(XEvent * event)	/* must be XButtonEvent */
+{
+    TScreen *screen = &term->screen;
+
+    /* Correct by half a width - we are acting on a boundary, not on a cell. */
+    return ((event->xbutton.x - OriginX(screen) + (FontWidth(screen) - 1) / 2)
+	    / FontWidth(screen));
+}
+
+static int
+ReadLineMovePoint(int col, int ldelta)
+{
+    TScreen *screen = &term->screen;
+    Char line[6];
+    int count = 0;
+
+    col += ldelta * (screen->max_col + 1) - screen->cur_col;
+    if (col == 0)
+	return 0;
+    if (screen->control_eight_bits) {
+	line[count++] = CSI;
+    } else {
+	line[count++] = ESC;
+	line[count++] = '[';	/* XXX maybe sometimes O is better? */
+    }
+    line[count++] = (col > 0 ? 'C' : 'D');
+    if (col < 0)
+	col = -col;
+    while (col--)
+	v_write(screen->respond, line, 3);
+    return 1;
+}
+
+static int
+ReadLineDelete(int r1, int c1, int r2, int c2)
+{
+    TScreen *screen = &term->screen;
+    int del;
+
+    del = c2 - c1 + (r2 - r1) * (screen->max_col + 1);
+    if (del <= 0)		/* Just in case... */
+	return 0;
+    while (del--)
+	v_write(screen->respond, "\177", 1);	/* XXX Sometimes "\08"? */
+    return 1;
+}
+#endif /* OPT_READLINE */
+
 /* ^XM-G<line+' '><col+' '> */
 void
 DiredButton(Widget w GCC_UNUSED,
@@ -641,9 +829,9 @@ DiredButton(Widget w GCC_UNUSED,
 	    String * params GCC_UNUSED,		/* selections */
 	    Cardinal * num_params GCC_UNUSED)
 {
-    register TScreen *screen = &term->screen;
+    TScreen *screen = &term->screen;
     Char Line[6];
-    register unsigned line, col;
+    unsigned line, col;
 
     if (event->type == ButtonPress || event->type == ButtonRelease) {
 	line = (event->xbutton.y - screen->border) / FontHeight(screen);
@@ -657,17 +845,77 @@ DiredButton(Widget w GCC_UNUSED,
     }
 }
 
-/* ^XM-G<line+' '><col+' '> */
+#if OPT_READLINE
+void
+ReadLineButton(Widget w GCC_UNUSED,
+	       XEvent * event,	/* must be XButtonEvent */
+	       String * params GCC_UNUSED,	/* selections */
+	       Cardinal * num_params GCC_UNUSED)
+{
+    TScreen *screen = &term->screen;
+    Char Line[6];
+    int line, col, ldelta = 0;
+
+    if (!(event->type == ButtonPress || event->type == ButtonRelease)
+	|| (screen->send_mouse_pos != MOUSE_OFF) || ExtendingSelection)
+	goto finish;
+    if (event->type == ButtonRelease) {
+	int delta;
+
+	if (lastButtonDownTime == (Time) 0)	/* first time and once in a blue moon */
+	    delta = screen->multiClickTime + 1;
+	else if (event->xbutton.time > lastButtonDownTime)	/* most of the time */
+	    delta = event->xbutton.time - lastButtonDownTime;
+	else			/* time has rolled over since lastButtonUpTime */
+	    delta = (((Time) ~ 0) - lastButtonDownTime) + event->xbutton.time;
+	if (delta > screen->multiClickTime)
+	    goto finish;	/* All this work for this... */
+    }
+    line = (event->xbutton.y - screen->border) / FontHeight(screen);
+    if (line != screen->cur_row) {
+	int l1, l2;
+
+	if (line < screen->cur_row)
+	    l1 = line, l2 = screen->cur_row;
+	else
+	    l2 = line, l1 = screen->cur_row;
+	l1--;
+	while (++l1 < l2)
+	    if (!ScrnTstWrapped(screen, l1))
+		goto finish;
+	/* Everything is on one "wrapped line" now */
+	ldelta = line - screen->cur_row;
+    }
+    /* Correct by half a width - we are acting on a boundary, not on a cell. */
+    col = (event->xbutton.x - OriginX(screen) + (FontWidth(screen) - 1) / 2)
+	/ FontWidth(screen) - screen->cur_col + ldelta * (screen->max_col + 1);
+    if (col == 0)
+	goto finish;
+    Line[0] = ESC;
+    /* XXX: sometimes it is better to send '['? */
+    Line[1] = 'O';
+    Line[2] = (col > 0 ? 'C' : 'D');
+    if (col < 0)
+	col = -col;
+    while (col--)
+	v_write(screen->respond, Line, 3);
+  finish:
+    if (event->type == ButtonRelease)
+	do_select_end(w, event, params, num_params, False);
+}
+#endif /* OPT_READLINE */
+
+/* repeats <ESC>n or <ESC>p */
 void
 ViButton(Widget w GCC_UNUSED,
 	 XEvent * event,	/* must be XButtonEvent */
 	 String * params GCC_UNUSED,	/* selections */
 	 Cardinal * num_params GCC_UNUSED)
 {
-    register TScreen *screen = &term->screen;
+    TScreen *screen = &term->screen;
     int pty = screen->respond;
     Char Line[6];
-    register int line;
+    int line;
 
     if (event->type == ButtonPress || event->type == ButtonRelease) {
 
@@ -699,7 +947,7 @@ HandleSelectExtend(Widget w,
 		   String * params GCC_UNUSED,
 		   Cardinal * num_params GCC_UNUSED)
 {
-    register TScreen *screen;
+    TScreen *screen;
     int row, col;
 
     if (!IsXtermWidget(w))
@@ -734,6 +982,11 @@ do_select_end(Widget w,
 	      Cardinal * num_params,
 	      Bool use_cursor_loc)
 {
+#if OPT_READLINE
+    int ldelta1, ldelta2;
+    TScreen *screen = &term->screen;
+#endif
+
     if (!IsXtermWidget(w))
 	return;
 
@@ -745,6 +998,20 @@ do_select_end(Widget w,
     case LEFTEXTENSION:
     case RIGHTEXTENSION:
 	EndExtend(w, event, params, *num_params, use_cursor_loc);
+#if OPT_READLINE
+	if (isClick1_clean(event)
+	    && SCREEN_FLAG(screen, click1_moves)
+	    && rowOnCurrentLine(eventRow(event), &ldelta1)) {
+	    ReadLineMovePoint(eventColBetween(event), ldelta1);
+	}
+	if (isDoubleClick3(event)
+	    && SCREEN_FLAG(screen, dclick3_deletes)
+	    && rowOnCurrentLine(startSRow, &ldelta1)
+	    && rowOnCurrentLine(endSRow, &ldelta2)) {
+	    ReadLineMovePoint(endSCol, ldelta2);
+	    ReadLineDelete(startSRow, startSCol, endSRow, endSCol);
+	}
+#endif /* OPT_READLINE */
 	break;
     }
 }
@@ -817,78 +1084,6 @@ UTF8toLatin1(Char * s, int len, unsigned long *result)
     *result = q - buffer;
     return buffer;
 }
-
-#if 0
-/* Eliminate all control characters from a UTF-8 string, doing
-   something reasonable with PS and LS */
-
-static Char *
-filterUTF8(Char * s, int len, int *len_return)
-{
-    Char *p = s;
-    Char *t;
-    Char *q;
-    unsigned codepoint;
-    int size;
-
-    t = (Char *) XtMalloc(len);
-    if (t == 0) {
-	TRACE(("Couldn't allocate target string\n"));
-	return 0;
-    }
-    q = t;
-
-    while (p < (s + len) && q < (t + len)) {
-	if ((*p & 0x80) == 0) {
-	    codepoint = *p & 0x7F;
-	    size = 1;
-	} else if ((*p & 0x60) == 0x40 && p < s + len - 1) {
-	    codepoint = ((p[0] & 0x1F) << 6) | (p[1] & 0x3F);
-	    size = 2;
-	} else if ((*p & 0x70) == 0x60 && p < s + len - 2) {
-	    codepoint = ((p[0] & 0x0F) << 12)
-		| ((p[1] & 0x3F) << 6)
-		| (p[2] & 0x3F);
-	    size = 3;
-	} else if ((*p & 0x78) == 0x70 && p < s + len - 3) {
-	    p += 4;		/* eliminate surrogates */
-	    continue;
-	} else if ((*p & 0x7C) == 0x78 && p < s + len - 4) {
-	    p += 5;
-	    continue;
-	} else if ((*p & 0x7E) == 0x7C && p < s + len - 5) {
-	    p += 6;
-	    continue;
-	} else {		/* wrong UTF-8?  Silently discard. */
-	    p++;
-	    continue;
-	}
-
-	if (codepoint == 0x2028) {
-	    /* line separator -- replace by NL */
-	    p += size;
-	    *q++ = 0x0A;
-	} else if (codepoint == 0x2029) {
-	    /* paragraph separator -- replace by NL NL */
-	    p += size;
-	    *q++ = 0x0A;
-	    if (q < t + len)
-		*q++ = 0x0A;
-	} else if (codepoint >= 0x202A && codepoint <= 0x202E) {
-	    /* ignore Unicode control characters; surrogates have already
-	       been eliminated */
-	    p += size;
-	} else {
-	    /* just copy the UTF-8 */
-	    while (size--)
-		*q++ = *p++;
-	}
-    }
-    *len_return = q - t;
-    return t;
-}
-#endif
-
 #endif /* OPT_WIDE_CHARS */
 
 static Atom *
@@ -1059,33 +1254,62 @@ GettingSelection(Display * dpy, Atom type, Char * line, int len)
 #define GettingSelection(dpy,type,line,len)	/* nothing */
 #endif
 
-#ifndef VMS
+#ifdef VMS
+#  define tty_vwrite(pty,lag,l)		tt_write(lag,l)
+#else /* !( VMS ) */
+#  define tty_vwrite(pty,lag,l)		v_write(pty,lag,l)
+#endif /* defined VMS */
+
 static void
-_WriteSelectionData(int pty, Char * line, int length)
+_qWriteSelectionData(TScreen * screen, Char * lag, int length)
+{
+#if OPT_READLINE
+    if (SCREEN_FLAG(screen, paste_quotes)) {
+	while (length--) {
+	    tty_vwrite(screen->respond, "\026", 1);	/* Control-V */
+	    tty_vwrite(screen->respond, lag++, 1);
+	}
+    } else
+#endif
+	tty_vwrite(screen->respond, lag, length);
+}
+
+static void
+_WriteSelectionData(TScreen * screen, Char * line, int length)
 {
     /* Write data to pty a line at a time. */
     /* Doing this one line at a time may no longer be necessary
        because v_write has been re-written. */
 
-    register Char *lag, *cp, *end;
+    Char *lag, *cp, *end;
 #if OPT_CHAT
     int first = 1;
 #endif
 
+    /* in the VMS version, if tt_pasting isn't set to TRUE then qio
+       reads aren't blocked and an infinite loop is entered, where the
+       pasted text shows up as new input, goes in again, shows up
+       again, ad nauseum. */
+#ifdef VMS
+    tt_pasting = TRUE;
+#endif
+
     end = &line[length];
     lag = line;
-    for (cp = line; cp != end; cp++) {
-	if (*cp == '\n') {
-	    *cp = '\r';
+    if (!SCREEN_FLAG(screen, paste_literal_nl)) {
+	for (cp = line; cp != end; cp++) {
+	    if (*cp == '\n') {
+		*cp = '\r';
 #if OPT_CHAT
-	    if (first) {
-		first = 0;
-		if (chat_mode)
-		    chat_flush(0);
-	    }
+		if (first) {
+		    first = 0;
+		    if (chat_mode)
+			chat_flush(0);
+		}
 #endif /* OPT_CHAT */
-	    v_write(pty, lag, cp - lag + 1);
-	    lag = cp + 1;
+		_qWriteSelectionData(screen, lag, cp - lag + 1);
+		lag = cp + 1;
+	    }
 	}
     }
     if (lag != end) {
@@ -1094,41 +1318,33 @@ _WriteSelectionData(int pty, Char * line, int length)
 	    chat_input(lag, end - lag);
 	else
 #endif /* OPT_CHAT */
-	v_write(pty, lag, end - lag);
+	_qWriteSelectionData(screen, lag, end - lag);
     }
-}
-
-#else /* VMS */
-
-static void
-_WriteSelectionData(int pty, Char * line, int length)
-{
-    /* in the VMS version, if tt_pasting isn't set to TRUE then qio
-       reads aren't blocked and an infinite loop is entered, where the
-       pasted text shows up as new input, goes in again, shows up
-       again, ad nauseum. */
-
-    register Char *lag, *cp, *end;
-
-    tt_pasting = TRUE;
-
-    end = &line[length];
-    lag = line;
-    for (cp = line; cp != end; cp++) {
-	if (*cp == '\n') {
-	    *cp = '\r';
-	    tt_write(lag, cp - lag + 1);
-	    lag = cp + 1;
-	}
-    }
-    if (lag != end) {
-	tt_write(lag, end - lag);
-    }
-
+#ifdef VMS
     tt_pasting = FALSE;
     tt_start_read();		/* reenable reads or a character may be lost */
+#endif
 }
-#endif /* VMS */
+
+#if OPT_READLINE
+static void
+_WriteKey(TScreen * screen, Char * in)
+{
+    char line[16];
+    int count = 0, length = strlen(in);
+
+    if (screen->control_eight_bits) {
+	line[count++] = CSI;
+    } else {
+	line[count++] = ESC;
+	line[count++] = '[';
+    }
+    while (length--)
+	line[count++] = *in++;
+    line[count++] = '~';
+    tty_vwrite(screen->respond, line, count);
+}
+#endif /* OPT_READLINE */
 
 /* SelectionReceived: stuff received selection text into pty */
 
@@ -1211,6 +1427,11 @@ SelectionReceived(Widget w,
 
     if (text_list != NULL && text_list_count != 0) {
 	int i;
+
+#if OPT_READLINE
+	if (SCREEN_FLAG(screen, paste_brackets))
+	    _WriteKey(screen, "200");
+#endif
 	for (i = 0; i < text_list_count; i++) {
 	    int len = strlen(text_list[i]);
 #if OPT_HANGUL
@@ -1221,9 +1442,12 @@ SelectionReceived(Widget w,
 		XtFree(tmp);
 	    }
 #endif /* OPT_HANGUL */
-	    _WriteSelectionData(((XtermWidget) w)->screen.respond,
-				(Char *) text_list[i], len);
+	    _WriteSelectionData(screen, (Char *) text_list[i], len);
 	}
+#if OPT_READLINE
+	if (SCREEN_FLAG(screen, paste_brackets))
+	    _WriteKey(screen, "201");
+#endif
 	XFreeStringList(text_list);
     } else
 	goto fail;
@@ -1249,8 +1473,24 @@ HandleInsertSelection(Widget w,
 		      String * params,	/* selections in precedence order */
 		      Cardinal * num_params)
 {
+#if OPT_READLINE
+    int ldelta;
+    TScreen *screen = &((XtermWidget) w)->screen;
+#endif
+
     if (SendMousePosition(w, event))
 	return;
+
+#if OPT_READLINE
+    if ((event->type == ButtonPress || event->type == ButtonRelease)
+    /* Disable on Shift-mouse, including the application-mouse modes */
+	&& !(KeyModifiers & ShiftMask)
+	&& (screen->send_mouse_pos == MOUSE_OFF)
+	&& SCREEN_FLAG(screen, paste_moves)
+	&& rowOnCurrentLine(eventRow(event), &ldelta))
+	ReadLineMovePoint(eventColBetween(event), ldelta);
+#endif /* OPT_READLINE */
+
     _GetSelection(w, event->xbutton.time, params, *num_params, NULL);
 }
 
@@ -1285,6 +1525,11 @@ do_select_start(Widget w,
 	return;
     selectUnit = EvalSelectUnit(event->xbutton.time, SELECTCHAR);
     replyToEmacs = FALSE;
+
+#if OPT_READLINE
+    lastButtonDownTime = event->xbutton.time;
+#endif
+
     StartSelect(startrow, startcol);
 }
 
@@ -1295,7 +1540,7 @@ HandleSelectStart(Widget w,
 		  String * params GCC_UNUSED,
 		  Cardinal * num_params GCC_UNUSED)
 {
-    register TScreen *screen;
+    TScreen *screen;
     int startrow, startcol;
 
     if (!IsXtermWidget(w))
@@ -1305,6 +1550,7 @@ HandleSelectStart(Widget w,
     firstValidRow = 0;
     lastValidRow = screen->max_row;
     PointToRowCol(event->xbutton.y, event->xbutton.x, &startrow, &startcol);
+
 #if OPT_HANGUL
     if ((event->xbutton.y - screen->border)/FontHeight(screen)
 	>= (lastValidRow + 1)) {
@@ -1314,9 +1560,16 @@ HandleSelectStart(Widget w,
 	    HandleChangeCode(w, event, params, num_params);
 	else if (han_is_kbd_button(startcol))
 	    HandleChangeKeyboard(w, event, params, num_params);
-    } else
+    }
+    else
 #endif /* OPT_HANGUL */
-    do_select_start(w, event, startrow, startcol);
+    {
+#if OPT_READLINE
+	ExtendingSelection = 0;
+#endif
+
+	do_select_start(w, event, startrow, startcol);
+    }
 }
 
 /* ARGSUSED */
@@ -1326,7 +1579,7 @@ HandleKeyboardSelectStart(Widget w,
 			  String * params GCC_UNUSED,
 			  Cardinal * num_params GCC_UNUSED)
 {
-    register TScreen *screen;
+    TScreen *screen;
 
     if (!IsXtermWidget(w))
 	return;
@@ -1336,7 +1589,7 @@ HandleKeyboardSelectStart(Widget w,
 }
 
 static void
-TrackDown(register XButtonEvent * event)
+TrackDown(XButtonEvent * event)
 {
     int startrow, startcol;
 
@@ -1505,10 +1758,21 @@ do_start_extend(Widget w,
 	return;
     firstValidRow = 0;
     lastValidRow = screen->max_row;
-    selectUnit = EvalSelectUnit(event->xbutton.time, selectUnit);
+#if OPT_READLINE
+    if ((KeyModifiers & ShiftMask)
+	|| event->xbutton.button != Button3
+	|| !(SCREEN_FLAG(screen, dclick3_deletes)))
+#endif
+	selectUnit = EvalSelectUnit(event->xbutton.time, selectUnit);
     replyToEmacs = FALSE;
 
-    if (numberOfClicks == 1) {
+#if OPT_READLINE
+    CheckSecondPress3(event);
+#endif
+
+    if (numberOfClicks == 1
+	|| (SCREEN_FLAG(screen, dclick3_deletes)	/* Dclick special */
+	    &&!(KeyModifiers & ShiftMask))) {
 	/* Save existing selection so we can reestablish it if the guy
 	   extends past the other end of the selection */
 	saveStartRRow = startERow = startRRow;
@@ -1545,6 +1809,11 @@ do_start_extend(Widget w,
 	endECol = col;
     }
     ComputeSelect(startERow, startECol, endERow, endECol, True);
+
+#if OPT_READLINE
+    if (startSRow != endSRow || startSCol != endSCol)
+	ExtendingSelection = 1;
+#endif
 }
 
 static void
@@ -1574,14 +1843,18 @@ ExtendExtend(int row, int col)
 	endECol = col;
     }
     ComputeSelect(startERow, startECol, endERow, endECol, False);
+
+#if OPT_READLINE
+    if (startSRow != endSRow || startSCol != endSCol)
+	ExtendingSelection = 1;
+#endif
 }
 
 void
-HandleStartExtend(
-		     Widget w,
-		     XEvent * event,	/* must be XButtonEvent* */
-		     String * params,	/* unused */
-		     Cardinal * num_params)	/* unused */
+HandleStartExtend(Widget w,
+		  XEvent * event,	/* must be XButtonEvent* */
+		  String * params,	/* unused */
+		  Cardinal * num_params)	/* unused */
 {
     do_start_extend(w, event, params, num_params, False);
 }
@@ -1596,11 +1869,11 @@ HandleKeyboardStartExtend(Widget w,
 }
 
 void
-ScrollSelection(register TScreen * screen, register int amount)
+ScrollSelection(TScreen * screen, int amount)
 {
-    register int minrow = -screen->savedlines - screen->topline;
-    register int maxrow = screen->max_row - screen->topline;
-    register int maxcol = screen->max_col;
+    int minrow = -screen->savedlines - screen->topline;
+    int maxrow = screen->max_row - screen->topline;
+    int maxcol = screen->max_col;
 
 #define scroll_update_one(row, col) \
 	row += amount; \
@@ -1666,8 +1939,8 @@ iswide(int i)
 #endif
 
 static void
-PointToRowCol(register int y,
-	      register int x,
+PointToRowCol(int y,
+	      int x,
 	      int *r,
 	      int *c)
 /* Convert pixel coordinates to character coordinates.
@@ -1675,8 +1948,8 @@ PointToRowCol(register int y,
    Columns are clipped between to be 0 or greater, but are not clipped to some
        maximum value. */
 {
-    register TScreen *screen = &term->screen;
-    register int row, col;
+    TScreen *screen = &term->screen;
+    int row, col;
 
     row = (y - screen->border) / FontHeight(screen);
     if (row < firstValidRow)
@@ -1711,11 +1984,11 @@ PointToRowCol(register int y,
 }
 
 static int
-LastTextCol(register int row)
+LastTextCol(int row)
 {
-    register TScreen *screen = &term->screen;
-    register int i;
-    register Char *ch;
+    TScreen *screen = &term->screen;
+    int i;
+    Char *ch;
 
     if ((row += screen->topline) + screen->savedlines >= 0) {
 	for (i = screen->max_col,
@@ -1882,10 +2155,9 @@ static int charClass[256] =
 /* *INDENT-ON* */
 
 int
-SetCharacterClassRange(
-			  register int low,	/* in range of [0..255] */
-			  register int high,
-			  register int value)	/* arbitrary */
+SetCharacterClassRange(int low,	/* in range of [0..255] */
+		       int high,
+		       int value)	/* arbitrary */
 {
 
     if (low < 0 || high > 255 || high < low)
@@ -1927,7 +2199,6 @@ class_of(TScreen * screen, int row, int col)
  * sets startSRow startSCol endSRow endSCol
  * ensuring that they have legal values
  */
-
 static void
 ComputeSelect(int startRow,
 	      int startCol,
@@ -1935,9 +2206,9 @@ ComputeSelect(int startRow,
 	      int endCol,
 	      Bool extend)
 {
-    register TScreen *screen = &term->screen;
-    register int length;
-    register int cclass;
+    TScreen *screen = &term->screen;
+    int length;
+    int cclass;
 
     TRACE(("ComputeSelect(startRow=%d, startCol=%d, endRow=%d, endCol=%d, %sextend)\n",
 	   startRow, startCol, endRow, endCol, extend ? "" : "no"));
@@ -2089,28 +2360,22 @@ ComputeSelect(int startRow,
 	break;
     }
 
-    if (startSRow < 0)
-	startSRow = 0;
-    if (startSCol < 0)
-	startSCol = 0;
-    if (endSRow < 0)
-	endSRow = 0;
-    if (endSCol < 0)
-	endSCol = 0;
+    /* check boundaries */
+    ScrollSelection(screen, 0);
 
     TrackText(startSRow, startSCol, endSRow, endSCol);
     return;
 }
 
 void
-TrackText(register int frow,
-	  register int fcol,
-	  register int trow,
-	  register int tcol)
+TrackText(int frow,
+	  int fcol,
+	  int trow,
+	  int tcol)
     /* Guaranteed (frow, fcol) <= (trow, tcol) */
 {
-    register int from, to;
-    register TScreen *screen = &term->screen;
+    int from, to;
+    TScreen *screen = &term->screen;
     int old_startrow, old_startcol, old_endrow, old_endcol;
 
     TRACE(("TrackText(frow=%d, fcol=%d, trow=%d, tcol=%d)\n",
@@ -2154,14 +2419,14 @@ TrackText(register int frow,
 }
 
 static void
-ReHiliteText(register int frow,
-	     register int fcol,
-	     register int trow,
-	     register int tcol)
+ReHiliteText(int frow,
+	     int fcol,
+	     int trow,
+	     int tcol)
     /* Guaranteed that (frow, fcol) <= (trow, tcol) */
 {
-    register TScreen *screen = &term->screen;
-    register int i;
+    TScreen *screen = &term->screen;
+    int i;
 
     if (frow < 0)
 	frow = fcol = 0;
@@ -2199,10 +2464,10 @@ SaltTextAway(int crow, int ccol, int row, int col,
     /* Guaranteed that (crow, ccol) <= (row, col), and that both points are valid
        (may have row = screen->max_row+1, col = 0) */
 {
-    register TScreen *screen = &term->screen;
-    register int i, j = 0;
+    TScreen *screen = &term->screen;
+    int i, j = 0;
     int eol;
-    char *line;
+    Char *line;
     Char *lp;
 
     if (crow == row && ccol > col) {
@@ -2241,9 +2506,9 @@ SaltTextAway(int crow, int ccol, int row, int col,
     /* now get some memory to save it in */
 
     if (screen->selection_size <= j) {
-	if ((line = (char *) malloc((unsigned) j + 1)) == 0)
+	if ((line = (Char *) malloc((unsigned) j + 1)) == 0)
 	    SysError(ERROR_BMALLOC2);
-	XtFree(screen->selection_data);
+	XtFree((char *) screen->selection_data);
 	screen->selection_data = line;
 	screen->selection_size = j + 1;
     } else {
@@ -2255,7 +2520,7 @@ SaltTextAway(int crow, int ccol, int row, int col,
 	return;
 
     line[j] = '\0';		/* make sure it is null terminated */
-    lp = (Char *) line;		/* lp points to where to save the text */
+    lp = line;			/* lp points to where to save the text */
     if (row == crow) {
 	lp = SaveText(screen, row, ccol, col, lp, &eol);
     } else {
@@ -2272,8 +2537,8 @@ SaltTextAway(int crow, int ccol, int row, int col,
     }
     *lp = '\0';			/* make sure we have end marked */
 
-    TRACE(("Salted TEXT:%.*s\n", (char *) lp - line, line));
-    screen->selection_length = ((char *) lp - line);
+    TRACE(("Salted TEXT:%.*s\n", lp - line, line));
+    screen->selection_length = (lp - line);
     _OwnSelection(term, params, num_params);
 }
 
@@ -2296,7 +2561,7 @@ _ConvertSelectionHelper(Widget w,
 
     screen = &((XtermWidget) w)->screen;
 
-    if (conversion_function(d, &screen->selection_data, 1,
+    if (conversion_function(d, (char **) &screen->selection_data, 1,
 			    conversion_style,
 			    &textprop) < Success)
 	return False;
@@ -2318,6 +2583,7 @@ ConvertSelection(Widget w,
 {
     Display *d = XtDisplay(w);
     TScreen *screen;
+    Boolean result = False;
 
     if (!IsXtermWidget(w))
 	return False;
@@ -2379,32 +2645,29 @@ ConvertSelection(Widget w,
 	XtFree((char *) std_targets);
 	*type = XA_ATOM;
 	*format = 32;
-	return True;
+	result = True;
     }
 #if OPT_WIDE_CHARS
-    if (screen->wide_chars && *target == XA_STRING) {
-	return
+    else if (screen->wide_chars && *target == XA_STRING) {
+	result =
 	    _ConvertSelectionHelper(w,
 				    type, value, length, format,
 				    Xutf8TextListToTextProperty,
 				    XStringStyle);
-    }
-    if (screen->wide_chars && *target == XA_UTF8_STRING(d)) {
-	return
+    } else if (screen->wide_chars && *target == XA_UTF8_STRING(d)) {
+	result =
 	    _ConvertSelectionHelper(w,
 				    type, value, length, format,
 				    Xutf8TextListToTextProperty,
 				    XUTF8StringStyle);
-    }
-    if (screen->wide_chars && *target == XA_TEXT(d)) {
-	return
+    } else if (screen->wide_chars && *target == XA_TEXT(d)) {
+	result =
 	    _ConvertSelectionHelper(w,
 				    type, value, length, format,
 				    Xutf8TextListToTextProperty,
 				    XStdICCTextStyle);
-    }
-    if (screen->wide_chars && *target == XA_COMPOUND_TEXT(d)) {
-	return
+    } else if (screen->wide_chars && *target == XA_COMPOUND_TEXT(d)) {
+	result =
 	    _ConvertSelectionHelper(w,
 				    type, value, length, format,
 				    Xutf8TextListToTextProperty,
@@ -2412,7 +2675,7 @@ ConvertSelection(Widget w,
     }
 #endif
 
-    if (*target == XA_STRING) {	/* not wide_chars */
+    else if (*target == XA_STRING) {	/* not wide_chars */
 	/* We can only reach this point if the selection requestor
 	   requested STRING before any of TEXT, COMPOUND_TEXT or
 	   UTF8_STRING.  We therefore assume that the requestor is not
@@ -2423,35 +2686,30 @@ ConvertSelection(Widget w,
 	*value = screen->selection_data;
 	*length = screen->selection_length;
 	*format = 8;
-	return True;
-    }
-
-    if (*target == XA_TEXT(d)) {	/* not wide_chars */
-	return
+	result = True;
+    } else if (*target == XA_TEXT(d)) {		/* not wide_chars */
+	result =
 	    _ConvertSelectionHelper(w,
 				    type, value, length, format,
 				    XmbTextListToTextProperty,
 				    XStdICCTextStyle);
-    }
-
-    if (*target == XA_COMPOUND_TEXT(d)) {	/* not wide_chars */
-	return
+    } else if (*target == XA_COMPOUND_TEXT(d)) {	/* not wide_chars */
+	result =
 	    _ConvertSelectionHelper(w,
 				    type, value, length, format,
 				    XmbTextListToTextProperty,
 				    XCompoundTextStyle);
     }
 #ifdef X_HAVE_UTF8_STRING
-    if (*target == XA_UTF8_STRING(d)) {		/* not wide_chars */
-	return
+    else if (*target == XA_UTF8_STRING(d)) {	/* not wide_chars */
+	result =
 	    _ConvertSelectionHelper(w,
 				    type, value, length, format,
 				    XmbTextListToTextProperty,
 				    XUTF8StringStyle);
     }
 #endif
-
-    if (*target == XA_LIST_LENGTH(d)) {
+    else if (*target == XA_LIST_LENGTH(d)) {
 	*value = XtMalloc(4);
 	if (sizeof(long) == 4)
 	     *(long *) *value = 1;
@@ -2462,10 +2720,8 @@ ConvertSelection(Widget w,
 	*type = XA_INTEGER;
 	*length = 1;
 	*format = 32;
-	return True;
-    }
-
-    if (*target == XA_LENGTH(d)) {
+	result = True;
+    } else if (*target == XA_LENGTH(d)) {
 	/* This value is wrong if we have UTF-8 text */
 	*value = XtMalloc(4);
 	if (sizeof(long) == 4)
@@ -2477,23 +2733,23 @@ ConvertSelection(Widget w,
 	*type = XA_INTEGER;
 	*length = 1;
 	*format = 32;
-	return True;
+	result = True;
+    } else if (XmuConvertStandardSelection(w,
+					   screen->selection_time, selection,
+					   target, type, (XPointer *) value,
+					   length, format)) {
+	result = True;
     }
 
-    if (XmuConvertStandardSelection(w, screen->selection_time, selection,
-				    target, type, (XPointer *) value,
-				    length, format))
-	return True;
-
     /* else */
-    return False;
+    return result;
 }
 
 static void
 LoseSelection(Widget w, Atom * selection)
 {
-    register TScreen *screen;
-    register Atom *atomP;
+    TScreen *screen;
+    Atom *atomP;
     Cardinal i;
 
     if (!IsXtermWidget(w))
@@ -2544,7 +2800,7 @@ SelectionDone(Widget w GCC_UNUSED,
 }
 
 static void
-_OwnSelection(register XtermWidget termw,
+_OwnSelection(XtermWidget termw,
 	      String * selections,
 	      Cardinal count)
 {
@@ -2605,10 +2861,11 @@ _OwnSelection(register XtermWidget termw,
 		 *   Robert Brady, 2000-09-05
 		 */
 		unsigned long length = termw->screen.selection_length;
-		Char *data = (Char *) termw->screen.selection_data;
+		Char *data = termw->screen.selection_data;
 		if_OPT_WIDE_CHARS((&(termw->screen)), {
 		    data = UTF8toLatin1(data, length, &length);
 		});
+		TRACE(("XStoreBuffer(%d)\n", cutbuffer));
 		XStoreBuffer(XtDisplay((Widget) termw),
 			     (char *) data, length, cutbuffer);
 	    }
@@ -2626,7 +2883,7 @@ _OwnSelection(register XtermWidget termw,
 }
 
 void
-DisownSelection(register XtermWidget termw)
+DisownSelection(XtermWidget termw)
 {
     Atom *atoms = termw->screen.selection_atoms;
     Cardinal count = termw->screen.selection_count;
@@ -2674,12 +2931,12 @@ DisownSelection(register XtermWidget termw)
 /* returns number of chars in line from scol to ecol out */
 /* ARGSUSED */
 static int
-Length(register TScreen * screen GCC_UNUSED,
-       register int row,
-       register int scol,
-       register int ecol)
+Length(TScreen * screen GCC_UNUSED,
+       int row,
+       int scol,
+       int ecol)
 {
-    register int lastcol = LastTextCol(row);
+    int lastcol = LastTextCol(row);
 
     if (ecol > lastcol)
 	ecol = lastcol;
@@ -2692,7 +2949,7 @@ SaveText(TScreen * screen,
 	 int row,
 	 int scol,
 	 int ecol,
-	 register Char * lp,	/* pointer to where to put the text */
+	 Char * lp,		/* pointer to where to put the text */
 	 int *eol)
 {
     int i = 0;
@@ -2800,7 +3057,7 @@ BtnCode(XButtonEvent * event, int button)
 #define MOUSE_LIMIT (255 - 32)
 
 static void
-EditorButton(register XButtonEvent * event)
+EditorButton(XButtonEvent * event)
 {
     TScreen *screen = &term->screen;
     int pty = screen->respond;
