@@ -1,10 +1,10 @@
 /*
- * $XFree86: xc/programs/xterm/fontutils.c,v 1.34 2002/05/24 20:14:22 keithp Exp $
+ * $XFree86: xc/programs/xterm/fontutils.c,v 1.37 2003/03/09 23:39:13 dickey Exp $
  */
 
 /************************************************************
 
-Copyright 1998-2001,2002 by Thomas E. Dickey
+Copyright 1998-2002,2003 by Thomas E. Dickey
 
                         All Rights Reserved
 
@@ -847,6 +847,8 @@ xtermLoadFont(TScreen * screen,
     screen->fnt_bold = bfs;
 #if OPT_WIDE_CHARS
     screen->fnt_dwd = wfs;
+    if (wbfs == NULL)
+	wbfs = wfs;
     screen->fnt_dwdb = wbfs;
 #endif
     screen->fnt_prop = proportional;
@@ -1014,7 +1016,7 @@ xtermComputeFontInfo(TScreen * screen,
 			XFT_FAMILY, XftTypeString, "mono",
 			XFT_SIZE, XftTypeInteger, term->misc.face_size,
 			XFT_SPACING, XftTypeInteger, XFT_MONO,
-			0);
+			(void *) 0);
 	match = XftFontMatch(dpy, DefaultScreen(dpy), pat, &result);
 	screen->renderFont = XftFontOpenPattern(dpy, match);
 	if (!screen->renderFont && match)
@@ -1023,7 +1025,7 @@ xtermComputeFontInfo(TScreen * screen,
 	    XftPatternBuild(pat,
 			    XFT_WEIGHT, XftTypeInteger, XFT_WEIGHT_BOLD,
 			    XFT_CHAR_WIDTH, XftTypeInteger, screen->renderFont->max_advance_width,
-			    0);
+			    (void *) 0);
 	    match = XftFontMatch(dpy, DefaultScreen(dpy), pat, &result);
 	    screen->renderFontBold = XftFontOpenPattern(dpy, match);
 	    if (!screen->renderFontBold && match)
