@@ -1,7 +1,7 @@
 /* $Xorg: menu.c,v 1.4 2001/02/09 02:06:03 xorgcvs Exp $ */
 /*
 
-Copyright 1999-2001,2002 by Thomas E. Dickey
+Copyright 1999-2002,2003 by Thomas E. Dickey
 
                         All Rights Reserved
 
@@ -45,17 +45,17 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/programs/xterm/menu.c,v 3.46 2002/04/28 19:04:21 dickey Exp $ */
+/* $XFree86: xc/programs/xterm/menu.c,v 3.51 2003/03/23 02:01:40 dickey Exp $ */
 
 #include <xterm.h>
 #include <data.h>
 #include <menu.h>
 #include <fontutils.h>
 
-#include <X11/StringDefs.h>
-#include <X11/Shell.h>
 #include <X11/Xmu/CharSet.h>
 
+#define app_con Xaw_app_con	/* quiet a warning from SimpleMenu.h */
+
 #if defined(HAVE_LIB_XAW)
 
 #include <X11/Xaw/SimpleMenu.h>
@@ -94,6 +94,8 @@ in this Software without prior written authorization from The Open Group.
 
 #endif
 
+#undef app_con
+
 #include <stdio.h>
 #include <signal.h>
 /* *INDENT-OFF* */
@@ -117,6 +119,7 @@ static void do_kill            PROTO_XT_CALLBACK_ARGS;
 static void do_marginbell      PROTO_XT_CALLBACK_ARGS;
 static void do_old_fkeys       PROTO_XT_CALLBACK_ARGS;
 static void do_print           PROTO_XT_CALLBACK_ARGS;
+static void do_print_redir     PROTO_XT_CALLBACK_ARGS;
 static void do_quit            PROTO_XT_CALLBACK_ARGS;
 static void do_redraw          PROTO_XT_CALLBACK_ARGS;
 static void do_reversevideo    PROTO_XT_CALLBACK_ARGS;
@@ -205,13 +208,15 @@ static void handle_vtshow      (Widget gw, Bool allowswitch);
 MenuEntry mainMenuEntries[] = {
     { "securekbd",	do_securekbd,	NULL },
     { "allowsends",	do_allowsends,	NULL },
+    { "redraw",		do_redraw,	NULL },
+    { "line1",		NULL,		NULL },
 #ifdef ALLOWLOGGING
     { "logging",	do_logging,	NULL },
 #endif
     { "print",		do_print,	NULL },
-    { "redraw",		do_redraw,	NULL },
-    { "line1",		NULL,		NULL },
-    { "8-bit control",	do_8bit_control, NULL },
+    { "print-redirect",	do_print_redir,	NULL },
+    { "line2",		NULL,		NULL },
+    { "8-bit control",	do_8bit_control,NULL },
     { "backarrow key",	do_backarrow,	NULL },
 #if OPT_NUM_LOCK
     { "num-lock",	do_num_lock,	NULL },
@@ -229,14 +234,14 @@ MenuEntry mainMenuEntries[] = {
 #if OPT_SUNPC_KBD
     { "sunKeyboard",	do_sun_kbd,	NULL },
 #endif
-    { "line2",		NULL,		NULL },
+    { "line3",		NULL,		NULL },
     { "suspend",	do_suspend,	NULL },
     { "continue",	do_continue,	NULL },
     { "interrupt",	do_interrupt,	NULL },
     { "hangup",		do_hangup,	NULL },
     { "terminate",	do_terminate,	NULL },
     { "kill",		do_kill,	NULL },
-    { "line3",		NULL,		NULL },
+    { "line4",		NULL,		NULL },
     { "quit",		do_quit,	NULL }};
 
 MenuEntry vtMenuEntries[] = {
@@ -454,10 +459,10 @@ create_menu(Widget w, XtermWidget xtw, MenuIndex num)
     return m;
 }
 
-static int
+static MenuIndex
 indexOfMenu(String menuName)
 {
-    int me;
+    MenuIndex me;
     switch (*menuName) {
     case 'm':
 	me = mainMenu;
@@ -474,7 +479,7 @@ indexOfMenu(String menuName)
 	break;
 #endif
     default:
-	me = -1;
+	me = noMenu;
     }
     return (me);
 }
@@ -491,7 +496,7 @@ domenu(Widget w GCC_UNUSED,
        Cardinal * param_count)	/* 0 or 1 */
 {
     TScreen *screen = &term->screen;
-    int me;
+    MenuIndex me;
     Boolean created = False;
     Widget mw;
 
@@ -500,14 +505,14 @@ domenu(Widget w GCC_UNUSED,
 	return False;
     }
 
-    if ((me = indexOfMenu(params[0])) < 0) {
+    if ((me = indexOfMenu(params[0])) == noMenu) {
 	Bell(XkbBI_MinorError, 0);
 	return False;
     }
 
-    if ((mw = obtain_menu(w, (MenuIndex) me)) == 0
+    if ((mw = obtain_menu(w, me)) == 0
 	|| sizeof_menu(w, me) == 0) {
-	mw = create_menu(w, term, (MenuIndex) me);
+	mw = create_menu(w, term, me);
 	created = (mw != 0);
     }
     if (mw == 0)
@@ -519,6 +524,7 @@ domenu(Widget w GCC_UNUSED,
 	    update_securekbd();
 	    update_allowsends();
 	    update_logging();
+	    update_print_redir();
 	    update_8bit_control();
 	    update_decbkm();
 	    update_num_lock();
@@ -530,14 +536,26 @@ domenu(Widget w GCC_UNUSED,
 				mainMenuEntries[mainMenu_8bit_ctrl].widget,
 				FALSE);
 	    }
-#if !defined(SIGTSTP) || defined(AMOEBA)
+#if !defined(SIGTSTP)
 	    set_sensitivity(mw,
 			    mainMenuEntries[mainMenu_suspend].widget, FALSE);
 #endif
-#if !defined(SIGCONT) || defined(AMOEBA)
+#if !defined(SIGCONT)
 	    set_sensitivity(mw,
 			    mainMenuEntries[mainMenu_continue].widget, FALSE);
 #endif
+#ifdef ALLOWLOGGING
+	    if (screen->inhibit & I_LOG) {
+		set_sensitivity(mw,
+				mainMenuEntries[mainMenu_logging].widget, FALSE);
+	    }
+#endif
+	    if (screen->inhibit & I_SIGNAL) {
+		int n;
+		for (n = (int) mainMenu_suspend; n <= (int) mainMenu_quit; ++n) {
+		    set_sensitivity(mw, mainMenuEntries[n].widget, FALSE);
+		}
+	    }
 	}
 	break;
 
@@ -572,6 +590,14 @@ domenu(Widget w GCC_UNUSED,
 	    } else
 		update_activeicon();
 #endif /* NO_ACTIVE_ICON */
+#if OPT_TEK4014
+	    if (screen->inhibit & I_TEK) {
+		int n;
+		for (n = (int) vtMenu_tekshow; n <= (int) vtMenu_vthide; ++n) {
+		    set_sensitivity(mw, vtMenuEntries[n].widget, FALSE);
+		}
+	    }
+#endif
 	}
 	break;
 
@@ -617,6 +643,8 @@ domenu(Widget w GCC_UNUSED,
 	}
 	break;
 #endif
+    default:
+	break;
     }
 
     return True;
@@ -758,6 +786,14 @@ do_print(Widget gw GCC_UNUSED,
     xtermPrintScreen(TRUE);
 }
 
+static void
+do_print_redir(Widget gw GCC_UNUSED,
+	       XtPointer closure GCC_UNUSED,
+	       XtPointer data GCC_UNUSED)
+{
+    setPrinterControlMode(term->screen.printer_controlmode ? 0 : 2);
+}
+
 static void
 do_redraw(Widget gw GCC_UNUSED,
 	  XtPointer closure GCC_UNUSED,
@@ -884,7 +920,7 @@ do_suspend(Widget gw,
 	   XtPointer closure GCC_UNUSED,
 	   XtPointer data GCC_UNUSED)
 {
-#if defined(SIGTSTP) && !defined(AMOEBA)
+#if defined(SIGTSTP)
     handle_send_signal(gw, SIGTSTP);
 #endif
 }
@@ -895,7 +931,7 @@ do_continue(Widget gw,
 	    XtPointer closure GCC_UNUSED,
 	    XtPointer data GCC_UNUSED)
 {
-#if defined(SIGCONT) && !defined(AMOEBA)
+#if defined(SIGCONT)
     handle_send_signal(gw, SIGCONT);
 #endif
 }
@@ -1163,7 +1199,7 @@ do_activeicon(Widget gw GCC_UNUSED,
 	term->misc.active_icon = !term->misc.active_icon;
 	XtVaSetValues(shell, XtNiconWindow,
 		      term->misc.active_icon ? screen->iconVwin.window : None,
-		      NULL);
+		      (XtPointer) 0);
 	update_activeicon();
     }
 }
@@ -1484,14 +1520,24 @@ HandleLogging(Widget w,
 
 /* ARGSUSED */
 void
-HandlePrint(Widget w,
-	    XEvent * event GCC_UNUSED,
-	    String * params GCC_UNUSED,
-	    Cardinal * param_count GCC_UNUSED)
+HandlePrintScreen(Widget w,
+		  XEvent * event GCC_UNUSED,
+		  String * params GCC_UNUSED,
+		  Cardinal * param_count GCC_UNUSED)
 {
     do_print(w, (XtPointer) 0, (XtPointer) 0);
 }
 
+/* ARGSUSED */
+void
+HandlePrintControlMode(Widget w,
+		       XEvent * event GCC_UNUSED,
+		       String * params GCC_UNUSED,
+		       Cardinal * param_count GCC_UNUSED)
+{
+    do_print_redir(w, (XtPointer) 0, (XtPointer) 0);
+}
+
 /* ARGSUSED */
 void
 HandleRedraw(Widget w,
@@ -2055,7 +2101,7 @@ SetItemSensitivity(Widget mi, XtArgVal val)
  */
 static void
 InitPopup(Widget gw,
-	  XtPointer closure GCC_UNUSED,
+	  XtPointer closure,
 	  XtPointer data GCC_UNUSED)
 {
     String params[2];
@@ -2080,12 +2126,12 @@ SetupShell(Widget * menus, MenuList * shell, Widget * menu_tops, int n, int m)
 				      simpleMenuWidgetClass,
 				      *menus,
 				      XtNgeometry, NULL,
-				      NULL);
+				      (XtPointer) 0);
 
     XtAddCallback(shell[n].w, XtNpopupCallback, InitPopup, menu_names[n].internal_name);
     XtVaGetValues(shell[n].w,
 		  XtNlabel, &external_name,
-		  NULL);
+		  (XtPointer) 0);
 
     TRACE(("...SetupShell(%s) -> %s -> %#lx\n",
 	   menu_names[n].internal_name,
@@ -2101,7 +2147,7 @@ SetupShell(Widget * menus, MenuList * shell, Widget * menu_tops, int n, int m)
 							  : 0),
 					   XtNmenuName, menu_names[n].internal_name,
 					   XtNlabel, external_name,
-					   NULL);
+					   (XtPointer) 0);
 }
 
 #endif
@@ -2125,7 +2171,7 @@ SetupMenus(Widget shell, Widget * forms, Widget * menus)
 #if OPT_TOOLBAR
     *forms = XtVaCreateManagedWidget("form",
 				     formWidgetClass, shell,
-				     NULL);
+				     (XtPointer) 0);
     xtermAddInput(*forms);
 
     /*
@@ -2141,7 +2187,7 @@ SetupMenus(Widget shell, Widget * forms, Widget * menus)
 				     XtNbottom, XawChainTop,
 				     XtNleft, XawChainLeft,
 				     XtNright, XawChainLeft,
-				     NULL);
+				     (XtPointer) 0);
 
     if (shell == toplevel) {	/* vt100 */
 	for (n = mainMenu; n <= fontMenu; n++) {
@@ -2167,7 +2213,7 @@ SetupMenus(Widget shell, Widget * forms, Widget * menus)
 
 #if OPT_NOW_CHAT_SCROLL_PATCH
 static void
-do_nowchat(Widget gw, XtPointer closure, data)
+do_nowchat(Widget gw, XtPointer closure, XtPointer data)
 {
     term->misc.han_now_chat = !term->misc.han_now_chat;
     update_nowchat();
@@ -2182,3 +2228,384 @@ HandleNowChat(Widget w, XEvent *event, String *params,
 }
 #endif /* OPT_NOW_CHAT_SCROLL_PATCH */
 
+void
+update_securekbd(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_securekbd].widget,
+		     term->screen.grabbedKbd);
+}
+
+void
+update_allowsends(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_allowsends].widget,
+		     term->screen.allowSendEvents);
+}
+
+#ifdef ALLOWLOGGING
+void
+update_logging(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_logging].widget,
+		     term->screen.logging);
+}
+#endif
+
+void
+update_print_redir(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_print_redir].widget,
+		     term->screen.printer_controlmode);
+}
+
+void
+update_8bit_control(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_8bit_ctrl].widget,
+		     term->screen.control_eight_bits);
+}
+
+void
+update_decbkm(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_backarrow].widget,
+		     term->keyboard.flags & MODE_DECBKM);
+}
+
+#if OPT_NUM_LOCK
+void
+update_num_lock(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_num_lock].widget,
+		     term->misc.real_NumLock);
+}
+
+void
+update_meta_esc(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_meta_esc].widget,
+		     term->screen.meta_sends_esc);
+}
+#endif
+
+void
+update_sun_fkeys(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_sun_fkeys].widget,
+		     term->keyboard.type == keyboardIsSun);
+}
+
+void
+update_old_fkeys(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_old_fkeys].widget,
+		     term->keyboard.type == keyboardIsLegacy);
+}
+
+void
+update_delete_del(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_delete_del].widget,
+		     xtermDeleteIsDEL());
+}
+
+#if OPT_SUNPC_KBD
+void
+update_sun_kbd(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_sun_kbd].widget,
+		     term->keyboard.type == keyboardIsVT220);
+}
+#endif
+
+#if OPT_HP_FUNC_KEYS
+void
+update_hp_fkeys(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_hp_fkeys].widget,
+		     term->keyboard.type == keyboardIsHP);
+}
+#endif
+
+#if OPT_SCO_FUNC_KEYS
+void
+update_sco_fkeys(void)
+{
+    update_menu_item(term->screen.mainMenu,
+		     mainMenuEntries[mainMenu_sco_fkeys].widget,
+		     term->keyboard.type == keyboardIsSCO);
+}
+#endif
+
+void
+update_scrollbar(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_scrollbar].widget,
+		     ScrollbarWidth(&term->screen));
+}
+
+void
+update_jumpscroll(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_jumpscroll].widget,
+		     term->screen.jumpscroll);
+}
+
+void
+update_reversevideo(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_reversevideo].widget,
+		     (term->misc.re_verse));
+}
+
+void
+update_autowrap(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_autowrap].widget,
+		     (term->flags & WRAPAROUND));
+}
+
+void
+update_reversewrap(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_reversewrap].widget,
+		     (term->flags & REVERSEWRAP));
+}
+
+void
+update_autolinefeed(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_autolinefeed].widget,
+		     (term->flags & LINEFEED));
+}
+
+void
+update_appcursor(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_appcursor].widget,
+		     (term->keyboard.flags & MODE_DECCKM));
+}
+
+void
+update_appkeypad(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_appkeypad].widget,
+		     (term->keyboard.flags & MODE_DECKPAM));
+}
+
+void
+update_scrollkey(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_scrollkey].widget,
+		     term->screen.scrollkey);
+}
+
+void
+update_scrollttyoutput(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_scrollttyoutput].widget,
+		     term->screen.scrollttyoutput);
+}
+
+void
+update_allow132(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_allow132].widget,
+		     term->screen.c132);
+}
+
+void
+update_cursesemul(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_cursesemul].widget,
+		     term->screen.curses);
+}
+
+void
+update_visualbell(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_visualbell].widget,
+		     term->screen.visualbell);
+}
+
+void
+update_poponbell(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_poponbell].widget,
+		     term->screen.poponbell);
+}
+
+void
+update_marginbell(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_marginbell].widget,
+		     term->screen.marginbell);
+}
+
+#if OPT_BLINK_CURS
+void
+update_cursorblink(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_cursorblink].widget,
+		     term->screen.cursor_blink);
+}
+#endif
+
+void
+update_altscreen(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_altscreen].widget,
+		     term->screen.alternate);
+}
+
+void
+update_titeInhibit(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_titeInhibit].widget,
+		     !(term->misc.titeInhibit));
+}
+
+#ifndef NO_ACTIVE_ICON
+void
+update_activeicon(void)
+{
+    update_menu_item(term->screen.vtMenu,
+		     vtMenuEntries[vtMenu_activeicon].widget,
+		     term->misc.active_icon);
+}
+#endif /* NO_ACTIVE_ICON */
+
+#if OPT_DEC_CHRSET
+void
+update_font_doublesize(void)
+{
+    update_menu_item(term->screen.fontMenu,
+		     fontMenuEntries[fontMenu_font_doublesize].widget,
+		     term->screen.font_doublesize);
+}
+#endif
+
+#if OPT_BOX_CHARS
+void
+update_font_boxchars(void)
+{
+    update_menu_item(term->screen.fontMenu,
+		     fontMenuEntries[fontMenu_font_boxchars].widget,
+		     term->screen.force_box_chars);
+}
+#endif
+
+#if OPT_DEC_SOFTFONT
+void
+update_font_loadable(void)
+{
+    update_menu_item(term->screen.fontMenu,
+		     fontMenuEntries[fontMenu_font_loadable].widget,
+		     term->misc.font_loadable);
+}
+#endif
+
+#if OPT_TEK4014
+void
+update_tekshow(void)
+{
+    if (!(term->screen.inhibit & I_TEK)) {
+	update_menu_item(term->screen.vtMenu,
+			 vtMenuEntries[vtMenu_tekshow].widget,
+			 term->screen.Tshow);
+    }
+}
+
+void
+update_vttekmode(void)
+{
+    if (!(term->screen.inhibit & I_TEK)) {
+	update_menu_item(term->screen.vtMenu,
+			 vtMenuEntries[vtMenu_tekmode].widget,
+			 term->screen.TekEmu);
+	update_menu_item(term->screen.tekMenu,
+			 tekMenuEntries[tekMenu_vtmode].widget,
+			 !term->screen.TekEmu);
+    }
+}
+
+void
+update_vtshow(void)
+{
+    if (!(term->screen.inhibit & I_TEK)) {
+	update_menu_item(term->screen.tekMenu,
+			 tekMenuEntries[tekMenu_vtshow].widget,
+			 term->screen.Vshow);
+    }
+}
+
+void
+set_vthide_sensitivity(void)
+{
+    if (!(term->screen.inhibit & I_TEK)) {
+	set_sensitivity(term->screen.vtMenu,
+			vtMenuEntries[vtMenu_vthide].widget,
+			term->screen.Tshow);
+    }
+}
+
+void
+set_tekhide_sensitivity(void)
+{
+    if (!(term->screen.inhibit & I_TEK)) {
+	set_sensitivity(term->screen.tekMenu,
+			tekMenuEntries[tekMenu_tekhide].widget,
+			term->screen.Vshow);
+    }
+}
+
+void
+set_tekfont_menu_item(int n, int val)
+{
+    if (!(term->screen.inhibit & I_TEK)) {
+	update_menu_item(term->screen.tekMenu,
+			 tekMenuEntries[FS2MI(n)].widget,
+			 (val));
+    }
+}
+#endif /* OPT_TEK4014 */
+
+void
+set_menu_font(int val)
+{
+    update_menu_item(term->screen.fontMenu,
+		     fontMenuEntries[term->screen.menu_font_number].widget,
+		     (val));
+}
+
