@@ -2,11 +2,11 @@
  *	$Xorg: misc.c,v 1.3 2000/08/17 19:55:09 cpqbld Exp $
  */
 
-/* $XFree86: xc/programs/xterm/misc.c,v 3.69 2002/06/01 00:54:49 dickey Exp $ */
+/* $XFree86: xc/programs/xterm/misc.c,v 3.77 2003/03/23 02:01:40 dickey Exp $ */
 
 /*
  *
- * Copyright 1999-2001,2002 by Thomas E. Dickey
+ * Copyright 1999-2002,2003 by Thomas E. Dickey
  *
  *                        All Rights Reserved
  *
@@ -69,7 +69,6 @@
 #include <X11/Xatom.h>
 #include <X11/cursorfont.h>
 
-#include <X11/Shell.h>
 #include <X11/Xmu/Error.h>
 #include <X11/Xmu/SysUtil.h>
 #include <X11/Xmu/WinUtil.h>
@@ -91,11 +90,6 @@
 #endif
 #endif
 
-#ifdef AMOEBA
-#include <amoeba.h>
-#include <module/proc.h>
-#endif
-
 #ifdef __UNIXOS2__
 #define environ gblenvp		/* circumvent a bug */
 #endif
@@ -127,6 +121,7 @@ static void DoSpecialEnterNotify(XEnterWindowEvent * ev);
 static void DoSpecialLeaveNotify(XEnterWindowEvent * ev);
 static void selectwindow(TScreen * screen, int flag);
 static void unselectwindow(TScreen * screen, int flag);
+static void Sleep(int msec);
 
 void
 do_xevents(void)
@@ -167,8 +162,15 @@ xevents(void)
     if ((input_mask & XtIMXEvent) != XtIMXEvent)
 	return;
     do {
-	if (waitingForTrackInfo)
+	/*
+	 * This check makes xterm hang when in mouse hilite tracking mode.
+	 * We simply ignore all events except for those not passed down to
+	 * this function, e.g., those handled in in_put().
+	 */
+	if (waitingForTrackInfo) {
+	    Sleep(10);
 	    return;
+	}
 	XtAppNextEvent(app_con, &event);
 	/*
 	 * Hack to get around problems with the toolkit throwing away
@@ -524,14 +526,15 @@ Bell(int which GCC_UNUSED, int percent)
 	lastBellTime = now_msecs;
     }
 
-    if (screen->visualbell)
+    if (screen->visualbell) {
 	VisualBell();
-    else
-#ifdef XKB
-	XkbStdBell(screen->display, TWindow(screen), percent, which);
+    } else {
+#if defined(HAVE_XKBBELL)
+	XkbBell(screen->display, VShellWindow, percent, which);
 #else
 	XBell(screen->display, percent);
 #endif
+    }
 
     if (screen->poponbell)
 	XRaiseWindow(screen->display, VShellWindow);
@@ -612,6 +615,31 @@ HandleBellPropertyChange(
     }
 }
 
+Window
+WMFrameWindow(XtermWidget termw)
+{
+    Window win_root, win_current, *children;
+    Window win_parent = 0;
+    unsigned int nchildren;
+
+    win_current = XtWindow(termw);
+
+    /* find the parent which is child of root */
+    do {
+	if (win_parent)
+	    win_current = win_parent;
+	XQueryTree((&termw->screen)->display,
+		   win_current,
+		   &win_root,
+		   &win_parent,
+		   &children,
+		   &nchildren);
+	XFree(children);
+    } while (win_root != win_parent);
+
+    return win_current;
+}
+
 #if OPT_MAXIMIZE
 /*ARGSUSED*/
 void
@@ -698,7 +726,6 @@ RequestMaximize(XtermWidget termw, int maximize)
 {
     register TScreen *screen = &termw->screen;
     XWindowAttributes win_attrs;
-    Position root_x, root_y;
     unsigned root_width, root_height;
 
     if (maximize) {
@@ -706,16 +733,15 @@ RequestMaximize(XtermWidget termw, int maximize)
 	if (QueryMaximize(screen, &root_width, &root_height)) {
 
 	    if (XGetWindowAttributes(screen->display,
-				     VShellWindow,
+				     WMFrameWindow(termw),
 				     &win_attrs)) {
-		XtTranslateCoords(toplevel, 0, 0, &root_x, &root_y);
 
 		if (screen->restore_data != True
 		    || screen->restore_width != root_width
 		    || screen->restore_height != root_height) {
 		    screen->restore_data = True;
-		    screen->restore_x = root_x - win_attrs.x;
-		    screen->restore_y = root_y - win_attrs.y;
+		    screen->restore_x = win_attrs.x;
+		    screen->restore_y = win_attrs.y;
 		    screen->restore_width = win_attrs.width;
 		    screen->restore_height = win_attrs.height;
 		    TRACE(("HandleMaximize: save window position %d,%d size %d,%d\n",
@@ -855,7 +881,7 @@ open_userfile(int uid, int gid, char *path, Boolean append)
 #else
     if ((access(path, F_OK) != 0 && (errno != ENOENT))
 	|| (!(creat_as(uid, gid, append, path, 0644)))
-	|| ((fd = open(path, O_WRONLY | O_APPEND, 0644)) < 0)) {
+	|| ((fd = open(path, O_WRONLY | O_APPEND)) < 0)) {
 	fprintf(stderr, "%s: cannot open %s: %d:%s\n",
 		xterm_name,
 		path,
@@ -1003,8 +1029,33 @@ StartLog(register TScreen * screen)
 	if (screen->logfile)
 	    free(screen->logfile);
 	if (log_default == NULL) {
+#if defined(HAVE_GETHOSTNAME) && defined(HAVE_STRFTIME)
+	    char log_def_name[512];	/* see sprintf below */
+	    char hostname[255 + 1];	/* Internet standard limit (RFC 1035):
+					   ``To simplify implementations, the
+					   total length of a domain name (i.e.,
+					   label octets and label length
+					   octets) is restricted to 255 octets
+					   or less.'' */
+	    char yyyy_mm_dd_hh_mm_ss[4 + 5 * (1 + 2) + 1];
+	    time_t now;
+	    struct tm *ltm;
+
+	    (void) time(&now);
+	    ltm = (struct tm *) localtime(&now);
+	    if ((gethostname(hostname, sizeof(hostname)) == 0) &&
+		(strftime(yyyy_mm_dd_hh_mm_ss,
+			  sizeof(yyyy_mm_dd_hh_mm_ss),
+			  "%Y.%m.%d.%H.%M.%S", ltm) > 0)) {
+		(void) sprintf(log_def_name, "Xterm.log.%.255s.%.20s.XXXXXX",
+			       hostname, yyyy_mm_dd_hh_mm_ss);
+	    }
+#else
+	    const char *log_def_name = "XtermLog.XXXXXX";
+#endif
 	    if ((log_default = x_strdup(log_def_name)) == NULL)
 		return;
+
 	    mktemp(log_default);
 	}
 	if ((screen->logfile = x_strdup(log_default)) == 0)
@@ -1054,7 +1105,7 @@ StartLog(register TScreen * screen)
 	    setgid(screen->gid);
 	    setuid(screen->uid);
 
-	    execl(shell, shell, "-c", &screen->logfile[1], 0);
+	    execl(shell, shell, "-c", &screen->logfile[1], (void *) 0);
 
 	    fprintf(stderr, "%s: Can't exec `%s'\n", xterm_name,
 		    &screen->logfile[1]);
@@ -1170,14 +1221,14 @@ find_closest_color(Display * display, Colormap cmap, XColor * def)
     double tmp, distance, closestDistance;
     int i, closest, numFound, cmap_size;
     XColor *colortable;
-    XVisualInfo template, *visInfoPtr;
+    XVisualInfo myTemplate, *visInfoPtr;
     char *found;
     int attempts;
 
-    template.visualid = XVisualIDFromVisual(DefaultVisual(display,
-							  XDefaultScreen(display)));
+    myTemplate.visualid = XVisualIDFromVisual(DefaultVisual(display,
+							    XDefaultScreen(display)));
     visInfoPtr = XGetVisualInfo(display, (long) VisualIDMask,
-				&template, &numFound);
+				&myTemplate, &numFound);
     if (numFound < 1) {
 	/* FindClosestColor couldn't lookup visual */
 	return FALSE;
@@ -1246,10 +1297,9 @@ find_closest_color(Display * display, Colormap cmap, XColor * def)
 }
 
 static Boolean
-AllocateAnsiColor(
-		     XtermWidget pTerm,
-		     ColorRes * res,
-		     char *spec)
+AllocateAnsiColor(XtermWidget pTerm,
+		  ColorRes * res,
+		  char *spec)
 {
     XColor def;
     register TScreen *screen = &pTerm->screen;
@@ -1259,7 +1309,9 @@ AllocateAnsiColor(
 	&& (XAllocColor(screen->display, cmap, &def)
 	    || find_closest_color(screen->display, cmap, &def))) {
 	SET_COLOR_RES(res, def.pixel);
-	TRACE(("AllocateAnsiColor %s (pixel %#lx)\n", spec, def.pixel));
+	TRACE(("AllocateAnsiColor[%d] %s (pixel %#lx)\n",
+	       (res - screen->Acolors), spec, def.pixel));
+	res->mode = True;
 	return (TRUE);
     }
     TRACE(("AllocateAnsiColor %s (failed)\n", spec));
@@ -1271,9 +1323,9 @@ Pixel
 xtermGetColorRes(ColorRes * res)
 {
     if (!res->mode) {
-	if (AllocateAnsiColor(term, res, res->resource)) {
-	    res->mode = True;
-	} else {
+	TRACE(("xtermGetColorRes for Acolors[%d]\n",
+	       res - term->screen.Acolors));
+	if (!AllocateAnsiColor(term, res, res->resource)) {
 	    res->value = term->screen.foreground;
 	    res->mode = -True;
 	    fprintf(stderr,
@@ -1314,12 +1366,15 @@ ChangeAnsiColorRequest(
 	}
 	if (!strcmp(name, "?"))
 	    ReportAnsiColorRequest(pTerm, color, final);
-	else if (!AllocateAnsiColor(pTerm, &(pTerm->screen.Acolors[color]), name))
-	    break;
-	/* FIXME:  free old color somehow?  We aren't for the other color
-	 * change style (dynamic colors).
-	 */
-	r = True;
+	else {
+	    TRACE(("ChangeAnsiColor for Acolors[%d]\n", color));
+	    if (!AllocateAnsiColor(pTerm, &(pTerm->screen.Acolors[color]), name))
+		break;
+	    /* FIXME:  free old color somehow?  We aren't for the other color
+	     * change style (dynamic colors).
+	     */
+	    r = True;
+	}
     }
     if (r)
 	ChangeAnsiColors(pTerm);
@@ -1511,16 +1566,29 @@ static struct {
     int len;
 } user_keys[MAX_UDK];
 
+/*
+ * Parse one nibble of a hex byte from the OSC string.  We have removed the
+ * string-terminator (replacing it with a null), so the only other delimiter
+ * that is expected is semicolon.  Ignore other characters (Ray Neuman says
+ * "real" terminals accept commas in the string definitions).
+ */
 static int
-hexvalue(int c)
+udk_value(char **cp)
 {
-    if (c >= '0' && c <= '9')
-	return c - '0';
-    if (c >= 'A' && c <= 'F')
-	return c - 'A' + 10;
-    if (c >= 'a' && c <= 'f')
-	return c - 'a' + 10;
-    return -1;
+    int c;
+
+    for (;;) {
+	if ((c = **cp) != '\0')
+	    *cp = *cp + 1;
+	if (c == ';' || c == '\0')
+	    return -1;
+	if (c >= '0' && c <= '9')
+	    return c - '0';
+	if (c >= 'A' && c <= 'F')
+	    return c - 'A' + 10;
+	if (c >= 'a' && c <= 'f')
+	    return c - 'a' + 10;
+    }
 }
 
 void
@@ -1725,19 +1793,18 @@ do_dcs(Char * dcsbuf, size_t dcslen)
 		reset_decudk();
 
 	    while (*cp) {
+		char *base = cp;
 		char *str = (char *) malloc(strlen(cp) + 2);
 		unsigned key = 0;
+		int lo, hi;
 		int len = 0;
 
 		while (isdigit(CharOf(*cp)))
 		    key = (key * 10) + (*cp++ - '0');
 		if (*cp == '/') {
 		    cp++;
-		    while (*cp != ';' && *cp != '\0') {
-			int hi = hexvalue(*cp++);
-			int lo = hexvalue(*cp++);
-			if (hi < 0 || lo < 0)
-			    return;
+		    while ((hi = udk_value(&cp)) >= 0
+			   && (lo = udk_value(&cp)) >= 0) {
 			str[len++] = (hi << 4) | lo;
 		    }
 		}
@@ -1751,6 +1818,8 @@ do_dcs(Char * dcsbuf, size_t dcslen)
 		}
 		if (*cp == ';')
 		    cp++;
+		if (cp == base)	/* badly-formed sequence - bail out */
+		    break;
 	    }
 	}
 	break;
@@ -1771,7 +1840,7 @@ static void
 ChangeGroup(String attribute, char *value)
 {
     Arg args[1];
-    char *name = (value != 0) ? (char *) value : "";
+    const char *name = (value != 0) ? (char *) value : "";
 
     TRACE(("ChangeGroup(attribute=%s, value=%s)\n", attribute, name));
 #if OPT_SAME_NAME
@@ -2055,26 +2124,86 @@ Panic(char *s GCC_UNUSED, int a GCC_UNUSED)
 char *
 SysErrorMsg(int n)
 {
-    register char *s = strerror(n);
-    return s ? s : "unknown error";
+    static char unknown[] = "unknown error";
+    char *s = strerror(n);
+    return s ? s : unknown;
 }
 
 void
 SysError(int i)
 {
+    static const char *table[] =
+    {
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+	,"main: ioctl() failed on FIONBIO"	/* 11 */
+	,"main: ioctl() failed on F_GETFL"	/* 12 */
+	,"main: ioctl() failed on F_SETFL"	/* 13 */
+	,"spawn: open() failed on /dev/tty"	/* 14 */
+	,"spawn: ioctl() failed on TIOCGETP"	/* 15 */
+	,0
+	,"spawn: ptsname() failed"	/* 17 */
+	,"spawn: open() failed on ptsname"	/* 18 */
+	,"spawn: ioctl() failed on I_PUSH/\"ptem\""	/* 19 */
+	,"spawn: ioctl() failed on I_PUSH/\"consem\""	/* 20 */
+	,"spawn: ioctl() failed on I_PUSH/\"ldterm\""	/* 21 */
+	,"spawn: ioctl() failed on I_PUSH/\"ttcompat\""		/* 22 */
+	,"spawn: ioctl() failed on TIOCSETP"	/* 23 */
+	,"spawn: ioctl() failed on TIOCSETC"	/* 24 */
+	,"spawn: ioctl() failed on TIOCSETD"	/* 25 */
+	,"spawn: ioctl() failed on TIOCSLTC"	/* 26 */
+	,"spawn: ioctl() failed on TIOCLSET"	/* 27 */
+	,"spawn: initgroups() failed"	/* 28 */
+	,"spawn: fork() failed"	/* 29 */
+	,"spawn: exec() failed"	/* 30 */
+	,0
+	,"get_pty: not enough ptys"	/* 32 */
+	,0
+	,"waiting for initial map"	/* 34 */
+	,"spawn: setuid() failed"	/* 35 */
+	,"spawn: can't initialize window"	/* 36 */
+	,0, 0, 0, 0, 0, 0, 0, 0, 0
+	,"spawn: ioctl() failed on TIOCKSET"	/* 46 */
+	,"spawn: ioctl() failed on TIOCKSETC"	/* 47 */
+	,"spawn: realloc of ttydev failed"	/* 48 */
+	,"luit: command-line malloc failed"	/* 49 */
+	,"in_put: select() failed"	/* 50 */
+	,0, 0, 0
+	,"VTInit: can't initialize window"	/* 54 */
+	,0, 0
+	,"HandleKeymapChange: malloc failed"	/* 57 */
+	,0, 0
+	,"Tinput: select() failed"	/* 60 */
+	,0, 0, 0
+	,"TekInit: can't initialize window"	/* 64 */
+	,0, 0, 0, 0, 0, 0
+	,"SaltTextAway: malloc() failed"	/* 71 */
+	,0, 0, 0, 0, 0, 0, 0, 0
+	,"StartLog: exec() failed"	/* 80 */
+	,0, 0
+	,"xerror: XError event"	/* 83 */
+	,"xioerror: X I/O error"	/* 84 */
+	,0, 0, 0, 0, 0
+	,"Alloc: calloc() failed on base"	/* 90 */
+	,"Alloc: calloc() failed on rows"	/* 91 */
+	,"ScreenResize: realloc() failed on alt base"	/* 92 */
+	,0, 0, 0
+	,"ScreenResize: malloc() or realloc() failed"	/* 96 */
+	,0, 0, 0, 0, 0
+	,"ScrnPointers: malloc/realloc() failed"	/* 102 */
+	,0, 0, 0, 0, 0, 0, 0
+	,"ScrollBarOn: realloc() failed on base"	/* 110 */
+	,"ScrollBarOn: realloc() failed on rows"	/* 111 */
+	,0, 0, 0, 0, 0, 0, 0, 0, 0
+	,"my_memmove: malloc/realloc failed"	/* 121 */
+    };
     int oerrno;
 
     oerrno = errno;
-    /* perror(3) write(2)s to file descriptor 2 */
     fprintf(stderr, "%s: Error %d, errno %d: ", xterm_name, i, oerrno);
     fprintf(stderr, "%s\n", SysErrorMsg(oerrno));
-    Cleanup(i);
-}
-
-void
-Error(int i)
-{
-    fprintf(stderr, "%s: Error %d\n", xterm_name, i);
+    if ((Cardinal) i < XtNumber(table) && table[i] != 0) {
+	fprintf(stderr, "Reason: %s\n", table[i]);
+    }
     Cleanup(i);
 }
 
@@ -2114,16 +2243,17 @@ Cleanup(int code)
 	    Sleep(10);
 	}
     }
+#if OPT_SESSION_MGT
+    if (resource.sessionMgt) {
+	XtVaSetValues(toplevel,
+		      XtNjoinSession, False,
+		      (XtPointer *) 0);
+    }
+#endif
 
     if (screen->pid > 1) {
 	(void) kill_process_group(screen->pid, SIGHUP);
     }
-#ifdef AMOEBA
-    if (!NULLPORT(&screen->proccap.cap_port))
-	(void) pro_stun(&screen->proccap, -1L);
-    cb_close(screen->tty_outq);
-    cb_close(screen->tty_inq);
-#endif
     Exit(code);
 }
 
@@ -2185,8 +2315,6 @@ xioerror(Display * dpy)
     return 0;			/* appease the compiler */
 }
 
-extern char *ProgramName;
-
 void
 xt_error(String message)
 {
@@ -2253,8 +2381,6 @@ set_vt_visibility(Boolean on)
 }
 
 #if OPT_TEK4014
-extern Atom wm_delete_window;	/* for ICCCM delete window */
-
 void
 set_tek_visibility(Boolean on)
 {
@@ -2358,3 +2484,130 @@ hide_tek_window(void)
 	switch_modes(True);	/* does longjmp to vt mode */
 }
 #endif /* OPT_TEK4014 */
+
+static const char *
+skip_punct(const char *s)
+{
+    while (*s == '-' || *s == '/' || *s == '+' || *s == '#' || *s == '%') {
+	++s;
+    }
+    return s;
+}
+
+static int
+cmp_options(const void *a, const void *b)
+{
+    return strcmp(skip_punct(((const OptionHelp *) a)->opt),
+		  skip_punct(((const OptionHelp *) b)->opt));
+}
+
+static int
+cmp_resources(const void *a, const void *b)
+{
+    return strcmp(((const XrmOptionDescRec *) a)->option,
+		  ((const XrmOptionDescRec *) b)->option);
+}
+
+XrmOptionDescRec *
+sortedOptDescs(XrmOptionDescRec * descs, Cardinal res_count)
+{
+    static XrmOptionDescRec *res_array = 0;
+
+    if (res_array == 0) {
+	Cardinal j;
+
+	/* make a sorted index to 'resources' */
+	res_array = (XrmOptionDescRec *) calloc(res_count, sizeof(*res_array));
+	for (j = 0; j < res_count; j++)
+	    res_array[j] = descs[j];
+	qsort(res_array, res_count, sizeof(*res_array), cmp_resources);
+    }
+    return res_array;
+}
+
+/*
+ * The first time this is called, construct sorted index to the main program's
+ * list of options, taking into account the on/off options which will be
+ * compressed into one token.  It's a lot simpler to do it this way than
+ * maintain the list in sorted form with lots of ifdef's.
+ */
+OptionHelp *
+sortedOpts(OptionHelp * options, XrmOptionDescRec * descs, Cardinal numDescs)
+{
+    static OptionHelp *opt_array = 0;
+
+    if (opt_array == 0) {
+	Cardinal opt_count, j, k;
+#if OPT_TRACE
+	XrmOptionDescRec *res_array = sortedOptDescs(descs, numDescs);
+	int code;
+	char *mesg;
+#else
+	(void) k;
+	(void) descs;
+	(void) numDescs;
+#endif
+
+	/* count 'options' and make a sorted index to it */
+	for (opt_count = 0; options[opt_count].opt != 0; ++opt_count) {
+	    ;
+	}
+	opt_array = (OptionHelp *) calloc(opt_count + 1, sizeof(OptionHelp));
+	for (j = 0; j < opt_count; j++)
+	    opt_array[j] = options[j];
+	qsort(opt_array, opt_count, sizeof(OptionHelp), cmp_options);
+
+	/* supply the "turn on/off" strings if needed */
+#if OPT_TRACE
+	for (j = 0; j < opt_count; j++) {
+	    if (!strncmp(opt_array[j].opt, "-/+", 3)) {
+		char *name = opt_array[j].opt + 3;
+		for (k = 0; k < numDescs; ++k) {
+		    char *value = res_array[k].value;
+		    if (res_array[k].option[0] == '-') {
+			code = -1;
+		    } else if (res_array[k].option[0] == '+') {
+			code = 1;
+		    } else {
+			code = 0;
+		    }
+		    if (x_strindex(opt_array[j].desc, "inhibit") != 0)
+			code = -code;
+		    if (code != 0
+			&& res_array[k].value != 0
+			&& !strcmp(name, res_array[k].option + 1)) {
+			if (((code < 0) && !strcmp(value, "on"))
+			    || ((code > 0) && !strcmp(value, "off"))
+			    || ((code > 0) && !strcmp(value, "0"))) {
+			    mesg = "turn on/off";
+			} else {
+			    mesg = "turn off/on";
+			}
+			if (strncmp(mesg, opt_array[j].desc, strlen(mesg))) {
+			    if (strncmp(opt_array[j].desc, "turn ", 5)) {
+				char *s = malloc(strlen(mesg)
+						 + 2
+						 + strlen(opt_array[j].desc));
+				if (s != 0) {
+				    sprintf(s, "%s %s", mesg, opt_array[j].desc);
+				    opt_array[j].desc = s;
+				}
+			    } else {
+				TRACE(("OOPS "));
+			    }
+			}
+			TRACE(("%s: %s %s: %s (%s)\n",
+			       mesg,
+			       res_array[k].option,
+			       res_array[k].value,
+			       opt_array[j].opt,
+			       opt_array[j].desc));
+			break;
+		    }
+		}
+	    }
+	}
+#endif
+    }
+    return opt_array;
+}
