@@ -5,7 +5,7 @@
 #ifndef lint
 static char *rid = "$XConsortium: main.c,v 1.227.1.2 95/06/29 18:13:15 kaleb Exp $";
 #endif /* lint */
-/* $XFree86: xc/programs/xterm/os2main.c,v 3.52 2002/06/01 00:54:49 dickey Exp $ */
+/* $XFree86: xc/programs/xterm/os2main.c,v 3.59 2003/03/23 02:01:40 dickey Exp $ */
 
 /***********************************************************
 
@@ -62,11 +62,11 @@ SOFTWARE.
 #include <os2.h>
 #define XTERM_MAIN
 
+#define RES_OFFSET(field)	XtOffsetOf(XTERM_RESOURCE, field)
+
 #include <version.h>
 #include <xterm.h>
 
-#include <X11/StringDefs.h>
-#include <X11/Shell.h>
 #include <X11/cursorfont.h>
 #ifdef I18N
 #include <X11/Xlocale.h>
@@ -94,15 +94,23 @@ SOFTWARE.
 #include <xstrings.h>
 #include <xterm_io.h>
 
+#if OPT_WIDE_CHARS
+#include <charclass.h>
+#include <wcwidth.h>
+#endif
+
 int
 setpgrp(pid_t pid, gid_t pgid)
 {
+    return 0;
 }
 
 int
 chown(const char *fn, pid_t pid, gid_t gid)
 {
+    return 0;
 }
+
 char *
 ttyname(int fd)
 {
@@ -118,10 +126,22 @@ static SIGNAL_T reapchild(int n);
 static int spawn(void);
 static void get_terminal(void);
 static void resize(TScreen * s, char *oldtc, char *newtc);
+static void set_owner(char *device, int uid, int gid, int mode);
 
 static Bool added_utmp_entry = False;
 
-static char **command_to_exec;
+/*
+** Ordinarily it should be okay to omit the assignment in the following
+** statement. Apparently the c89 compiler on AIX 4.1.3 has a bug, or does
+** it? Without the assignment though the compiler will init command_to_exec
+** to 0xffffffff instead of NULL; and subsequent usage, e.g. in spawn() to
+** SEGV.
+*/
+static char **command_to_exec = NULL;
+
+#if OPT_LUIT_PROG
+static char **command_to_exec_with_luit = NULL;
+#endif
 
 /* The following structures are initialized in main() in order
 ** to eliminate any assumptions about the internal order of their
@@ -230,87 +250,38 @@ static Atom mit_console;
 static int tslot;
 static jmp_buf env;
 
-char *ProgramName;
-
-static struct _resource {
-    char *xterm_name;
-    char *icon_geometry;
-    char *title;
-    char *icon_name;
-    char *term_name;
-    char *tty_modes;
-    Boolean hold_screen;	/* true if we keep window open  */
-    Boolean utmpInhibit;
-    Boolean messages;
-    Boolean sunFunctionKeys;	/* %%% should be widget resource? */
-#if OPT_SUNPC_KBD
-    Boolean sunKeyboard;
-#endif
-#if OPT_HP_FUNC_KEYS
-    Boolean hpFunctionKeys;
-#endif
-    Boolean wait_for_map;
-    Boolean useInsertMode;
-#if OPT_ZICONBEEP
-    int zIconBeep;		/* beep level when output while iconified */
-#endif
-#if OPT_SAME_NAME
-    Boolean sameName;		/* Don't change the title or icon name if it is
-				 * the same.  This prevents flicker on the
-				 * screen at the cost of an extra request to
-				 * the server.
-				 */
-#endif
-} resource;
-
 /* used by VT (charproc.c) */
 
-#define offset(field)	XtOffsetOf(struct _resource, field)
-
 static XtResource application_resources[] =
 {
-    {"name", "Name", XtRString, sizeof(char *),
-     offset(xterm_name), XtRString, DFT_TERMTYPE},
-    {"iconGeometry", "IconGeometry", XtRString, sizeof(char *),
-     offset(icon_geometry), XtRString, (caddr_t) NULL},
-    {XtNtitle, XtCTitle, XtRString, sizeof(char *),
-     offset(title), XtRString, (caddr_t) NULL},
-    {XtNiconName, XtCIconName, XtRString, sizeof(char *),
-     offset(icon_name), XtRString, (caddr_t) NULL},
-    {"termName", "TermName", XtRString, sizeof(char *),
-     offset(term_name), XtRString, (caddr_t) NULL},
-    {"ttyModes", "TtyModes", XtRString, sizeof(char *),
-     offset(tty_modes), XtRString, (caddr_t) NULL},
-    {"hold", "Hold", XtRBoolean, sizeof(Boolean),
-     offset(hold_screen), XtRString, "false"},
-    {"utmpInhibit", "UtmpInhibit", XtRBoolean, sizeof(Boolean),
-     offset(utmpInhibit), XtRString, "false"},
-    {"messages", "Messages", XtRBoolean, sizeof(Boolean),
-     offset(messages), XtRString, "true"},
-    {"sunFunctionKeys", "SunFunctionKeys", XtRBoolean, sizeof(Boolean),
-     offset(sunFunctionKeys), XtRString, "false"},
+    Sres("name", "Name", xterm_name, DFT_TERMTYPE),
+    Sres("iconGeometry", "IconGeometry", icon_geometry, NULL),
+    Sres(XtNtitle, XtCTitle, title, NULL),
+    Sres(XtNiconName, XtCIconName, icon_name, NULL),
+    Sres("termName", "TermName", term_name, NULL),
+    Sres("ttyModes", "TtyModes", tty_modes, NULL),
+    Bres("hold", "Hold", hold_screen, FALSE),
+    Bres("utmpInhibit", "UtmpInhibit", utmpInhibit, FALSE),
+    Bres("messages", "Messages", messages, TRUE),
+    Bres("sunFunctionKeys", "SunFunctionKeys", sunFunctionKeys, FALSE),
 #if OPT_SUNPC_KBD
-    {"sunKeyboard", "SunKeyboard", XtRBoolean, sizeof(Boolean),
-     offset(sunKeyboard), XtRString, "false"},
+    Bres("sunKeyboard", "SunKeyboard", sunKeyboard, FALSE),
 #endif
 #if OPT_HP_FUNC_KEYS
-    {"hpFunctionKeys", "HpFunctionKeys", XtRBoolean, sizeof(Boolean),
-     offset(hpFunctionKeys), XtRString, "false"},
+    Bres("hpFunctionKeys", "HpFunctionKeys", hpFunctionKeys, FALSE),
 #endif
-    {"waitForMap", "WaitForMap", XtRBoolean, sizeof(Boolean),
-     offset(wait_for_map), XtRString, "false"},
-    {"useInsertMode", "UseInsertMode", XtRBoolean, sizeof(Boolean),
-     offset(useInsertMode), XtRString, "false"},
+    Bres("waitForMap", "WaitForMap", wait_for_map, FALSE),
+    Bres("useInsertMode", "UseInsertMode", useInsertMode, FALSE),
 #if OPT_ZICONBEEP
-    {"zIconBeep", "ZIconBeep", XtRInt, sizeof(int),
-     offset(zIconBeep), XtRImmediate, 0},
+    Ires("zIconBeep", "ZIconBeep", zIconBeep, 0),
 #endif
 #if OPT_SAME_NAME
-    {"sameName", "SameName", XtRBoolean, sizeof(Boolean),
-     offset(sameName), XtRString, "true"},
+    Bres("sameName", "SameName", sameName, TRUE),
+#endif
+#if OPT_SESSION_MGT
+    Bres("sessionMgt", "SessionMgt", sessionMgt, TRUE),
 #endif
 };
-#undef offset
 
 static char *fallback_resources[] =
 {
@@ -388,6 +359,10 @@ static XrmOptionDescRec optionDescList[] = {
 {"+hold",	"*hold",	XrmoptionNoArg,		(caddr_t) "off"},
 {"-j",		"*jumpScroll",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+j",		"*jumpScroll",	XrmoptionNoArg,		(caddr_t) "off"},
+#if OPT_C1_PRINT
+{"-k8",		"*allowC1Printable", XrmoptionNoArg,	(caddr_t) "on"},
+{"+k8",		"*allowC1Printable", XrmoptionNoArg,	(caddr_t) "off"},
+#endif
 /* parse logging options anyway for compatibility */
 {"-l",		"*logging",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+l",		"*logging",	XrmoptionNoArg,		(caddr_t) "off"},
@@ -437,6 +412,12 @@ static XrmOptionDescRec optionDescList[] = {
 {"-u8",		"*utf8",	XrmoptionNoArg,		(caddr_t) "2"},
 {"+u8",		"*utf8",	XrmoptionNoArg,		(caddr_t) "0"},
 #endif
+#if OPT_LUIT_PROG
+{"-lc",		"*locale",	XrmoptionNoArg,		(caddr_t) "True"},
+{"+lc",		"*locale",	XrmoptionNoArg,		(caddr_t) "False"},
+{"-lcc",	"*localeFilter",XrmoptionSepArg,	(caddr_t) NULL},
+{"-en",		"*locale",	XrmoptionSepArg,	(caddr_t) NULL},
+#endif
 {"-ulc",	"*colorULMode",	XrmoptionNoArg,		(caddr_t) "off"},
 {"+ulc",	"*colorULMode",	XrmoptionNoArg,		(caddr_t) "on"},
 {"-ut",		"*utmpInhibit",	XrmoptionNoArg,		(caddr_t) "on"},
@@ -450,23 +431,30 @@ static XrmOptionDescRec optionDescList[] = {
 #if OPT_WIDE_CHARS
 {"-wc",		"*wideChars",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+wc",		"*wideChars",	XrmoptionNoArg,		(caddr_t) "off"},
+{"-cjk_width", "*cjkWidth",	XrmoptionNoArg,		(caddr_t) "on"},
+{"+cjk_width", "*cjkWidth",	XrmoptionNoArg,		(caddr_t) "off"},
 #endif
 {"-wf",		"*waitForMap",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+wf",		"*waitForMap",	XrmoptionNoArg,		(caddr_t) "off"},
 #if OPT_ZICONBEEP
-{"-ziconbeep",  "*zIconBeep",   XrmoptionSepArg,        (caddr_t) NULL},
+{"-ziconbeep",	"*zIconBeep",	XrmoptionSepArg,	(caddr_t) NULL},
 #endif
 #if OPT_SAME_NAME
 {"-samename",	"*sameName",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+samename",	"*sameName",	XrmoptionNoArg,		(caddr_t) "off"},
 #endif
+#if OPT_SESSION_MGT
+{"-sm",		"*sessionMgt",	XrmoptionNoArg,		(caddr_t) "on"},
+{"+sm",		"*sessionMgt",	XrmoptionNoArg,		(caddr_t) "off"},
+#endif
 /* options that we process ourselves */
 {"-help",	NULL,		XrmoptionSkipNArgs,	(caddr_t) NULL},
 {"-version",	NULL,		XrmoptionSkipNArgs,	(caddr_t) NULL},
 {"-class",	NULL,		XrmoptionSkipArg,	(caddr_t) NULL},
 {"-e",		NULL,		XrmoptionSkipLine,	(caddr_t) NULL},
+{"-into",	NULL,		XrmoptionSkipArg,	(caddr_t) NULL},
 /* bogus old compatibility stuff for which there are
-   standard XtAppInitialize options now */
+   standard XtOpenApplication options now */
 {"%",		"*tekGeometry",	XrmoptionStickyArg,	(caddr_t) NULL},
 {"#",		".iconGeometry",XrmoptionStickyArg,	(caddr_t) NULL},
 {"-T",		".title",	XrmoptionSepArg,	(caddr_t) NULL},
@@ -478,7 +466,7 @@ static XrmOptionDescRec optionDescList[] = {
 {"-w",		".borderWidth", XrmoptionSepArg,	(caddr_t) NULL},
 };
 
-static OptionHelp options[] = {
+static OptionHelp xtermOptions[] = {
 { "-version",              "print the version number" },
 { "-help",                 "print out this message" },
 { "-display displayname",  "X server to contact" },
@@ -490,7 +478,7 @@ static OptionHelp options[] = {
 { "-bw number",            "border width in pixels" },
 { "-fn fontname",          "normal text font" },
 { "-fb fontname",          "bold text font" },
-{ "-/+fbb",                "turn on/off bold font's box checking"},
+{ "-/+fbb",                "turn on/off normal/bold font comparison inhibit"},
 { "-/+fbx",                "turn off/on linedrawing characters"},
 #ifdef XRENDERFONT
 { "-fa pattern",           "FreeType font-selection pattern" },
@@ -505,16 +493,16 @@ static OptionHelp options[] = {
 { "-class string",         "class string (XTerm)" },
 { "-title string",         "title string" },
 { "-xrm resourcestring",   "additional resource specifications" },
-{ "-/+132",                "turn on/off column switch inhibiting" },
+{ "-/+132",                "turn on/off 80/132 column switching" },
 { "-/+ah",                 "turn on/off always highlight" },
 #ifndef NO_ACTIVE_ICON
-{ "-/+ai",		   "turn on/off active icon" },
-{ "-fi fontname",	   "icon font for active icon" },
+{ "-/+ai",                 "turn off/on active icon" },
+{ "-fi fontname",          "icon font for active icon" },
 #endif /* NO_ACTIVE_ICON */
 { "-b number",             "internal border in pixels" },
-{ "-/+bc",		   "turn on/off text cursor blinking" },
-{ "-bcf milliseconds",	   "time text cursor is off when blinking"},
-{ "-bcn milliseconds",	   "time text cursor is on when blinking"},
+{ "-/+bc",                 "turn on/off text cursor blinking" },
+{ "-bcf milliseconds",     "time text cursor is off when blinking"},
+{ "-bcn milliseconds",     "time text cursor is on when blinking"},
 { "-/+bdc",                "turn off/on display of bold as color"},
 { "-/+cb",                 "turn on/off cut-to-beginning-of-line inhibit" },
 { "-cc classrange",        "specify additional character classes" },
@@ -522,16 +510,19 @@ static OptionHelp options[] = {
 { "-/+cn",                 "turn on/off cut newline inhibit" },
 { "-cr color",             "text cursor color" },
 { "-/+cu",                 "turn on/off curses emulation" },
-{ "-/+dc",		   "turn off/on dynamic color selection" },
+{ "-/+dc",                 "turn off/on dynamic color selection" },
 #if OPT_HIGHLIGHT_COLOR
-{ "-hc",		   "selection background color" },
+{ "-hc color",             "selection background color" },
 #endif
 #if OPT_HP_FUNC_KEYS
 { "-/+hf",                 "turn on/off HP Function Key escape codes" },
 #endif
-{ "-/+hold",		   "turn on/off logic that retains window after exit" },
-{ "-/+im",		   "use insert mode for TERMCAP" },
+{ "-/+hold",               "turn on/off logic that retains window after exit" },
+{ "-/+im",                 "use insert mode for TERMCAP" },
 { "-/+j",                  "turn on/off jump scroll" },
+#if OPT_C1_PRINT
+{ "-/+k8",                 "turn on/off C1-printable classification"},
+#endif
 #ifdef ALLOWLOGGING
 { "-/+l",                  "turn on/off logging" },
 { "-lf filename",          "logging filename" },
@@ -542,10 +533,10 @@ static OptionHelp options[] = {
 { "-/+ls",                 "turn on/off login shell" },
 { "-/+mb",                 "turn on/off margin bell" },
 { "-mc milliseconds",      "multiclick time in milliseconds" },
-{ "-/+mesg",		   "forbid/allow messages" },
+{ "-/+mesg",               "forbid/allow messages" },
 { "-ms color",             "pointer color" },
 { "-nb number",            "margin bell in characters from right end" },
-{ "-/+nul",                "turn on/off display of underlining" },
+{ "-/+nul",                "turn off/on display of underlining" },
 { "-/+aw",                 "turn on/off auto wraparound" },
 { "-/+pc",                 "turn on/off PC-style bold colors" },
 { "-/+rw",                 "turn on/off reverse wraparound" },
@@ -555,7 +546,7 @@ static OptionHelp options[] = {
 { "-rightbar",             "force scrollbar right (default left)" },
 { "-leftbar",              "force scrollbar left" },
 #endif
-{ "-/+rvc",		   "turn off/on display of reverse as color" },
+{ "-/+rvc",                "turn off/on display of reverse as color" },
 { "-/+sf",                 "turn on/off Sun Function Key escape codes" },
 { "-/+si",                 "turn on/off scroll-on-tty-output inhibit" },
 { "-/+sk",                 "turn on/off scroll-on-keypress" },
@@ -572,12 +563,17 @@ static OptionHelp options[] = {
 #if OPT_WIDE_CHARS
 { "-/+u8",                 "turn on/off UTF-8 mode (implies wide-characters)" },
 #endif
+#if OPT_LUIT_PROG
+{ "-/+lc",                 "turn on/off locale mode using luit" },
+{ "-lcc path",             "filename of locale converter (" DEFLOCALEFILTER ")" },
+#endif
 { "-/+ulc",                "turn off/on display of underline as color" },
 { "-/+ut",                 "turn on/off utmp inhibit (not supported)" },
 { "-/+vb",                 "turn on/off visual bell" },
 { "-/+pob",                "turn on/off pop on bell" },
 #if OPT_WIDE_CHARS
 { "-/+wc",                 "turn on/off wide-character mode" },
+{ "-/+cjk_width",          "turn on/off legacy CJK width convention" },
 #endif
 { "-/+wf",                 "turn on/off wait for map before command exec" },
 { "-e command args ...",   "command to execute" },
@@ -589,12 +585,16 @@ static OptionHelp options[] = {
 { "-n string",             "icon name for window" },
 { "-C",                    "intercept console messages" },
 { "-Sccn",                 "slave mode on \"ttycc\", file descriptor \"n\"" },
+{ "-into windowId",        "use the window id given to -into as the parent window rather than the default root window" },
 #if OPT_ZICONBEEP
 { "-ziconbeep percent",    "beep and flag icon of window having hidden output" },
 #endif
 #if OPT_SAME_NAME
 { "-/+samename",           "turn on/off the no-flicker option for title and icon name" },
 #endif
+#if OPT_SESSION_MGT
+{ "-/+sm",                 "turn on/off the session-management support" },
+#endif
 { NULL, NULL }};
 /* *INDENT-ON* */
 
@@ -622,6 +622,69 @@ static char *message[] =
     "will be started.  Options that start with a plus sign (+) restore the default.",
     NULL};
 
+/*
+ * Decode a key-definition.  This combines the termcap and ttyModes, for
+ * comparison.  Note that octal escapes in ttyModes are done by the normal
+ * resource translation.  Also, ttyModes allows '^-' as a synonym for disabled.
+ */
+static int
+decode_keyvalue(char **ptr, int termcap)
+{
+    char *string = *ptr;
+    int value = -1;
+
+    TRACE(("...decode '%s'\n", string));
+    if (*string == '^') {
+	switch (*++string) {
+	case '?':
+	    value = A2E(127);
+	    break;
+	case '-':
+	    if (!termcap) {
+		errno = 0;
+#if defined(_POSIX_VDISABLE) && defined(HAVE_UNISTD_H)
+		value = _POSIX_VDISABLE;
+#endif
+#if defined(_PC_VDISABLE)
+		if (value == -1) {
+		    value = fpathconf(0, _PC_VDISABLE);
+		    if (value == -1) {
+			if (errno != 0)
+			    break;	/* skip this (error) */
+			value = 0377;
+		    }
+		}
+#elif defined(VDISABLE)
+		if (value == -1)
+		    value = VDISABLE;
+#endif
+		break;
+	    }
+	    /* FALLTHRU */
+	default:
+	    value = CONTROL(*string);
+	    break;
+	}
+	++string;
+    } else if (termcap && (*string == '\\')) {
+	char *d;
+	int temp = strtol(string + 1, &d, 8);
+	if (temp > 0 && d != string) {
+	    value = temp;
+	    string = d;
+	}
+    } else {
+	value = CharOf(*string);
+	++string;
+    }
+    *ptr = string;
+    return value;
+}
+
+/*
+ * If we're linked to terminfo, tgetent() will return an empty buffer.  We
+ * cannot use that to adjust the $TERMCAP variable.
+ */
 static Boolean
 get_termcap(char *name, char *buffer, char *resized)
 {
@@ -659,6 +722,7 @@ static void
 Syntax(char *badOption)
 {
     OptionHelp *opt;
+    OptionHelp *list = sortedOpts(xtermOptions, optionDescList, XtNumber(optionDescList));
     int col;
 
     fprintf(stderr, "%s:  bad command line option \"%s\"\r\n\n",
@@ -666,7 +730,7 @@ Syntax(char *badOption)
 
     fprintf(stderr, "usage:  %s", ProgramName);
     col = 8 + strlen(ProgramName);
-    for (opt = options; opt->opt; opt++) {
+    for (opt = list; opt->opt; opt++) {
 	int len = 3 + strlen(opt->opt);		/* space [ string ] */
 	if (col + len > 79) {
 	    fprintf(stderr, "\r\n   ");		/* 3 spaces */
@@ -692,13 +756,14 @@ static void
 Help(void)
 {
     OptionHelp *opt;
+    OptionHelp *list = sortedOpts(xtermOptions, optionDescList, XtNumber(optionDescList));
     char **cpp;
 
     fprintf(stderr,
 	    "%s(%d) usage:\n    %s [-options ...] [-e command args]\n\n",
 	    XFREE86_VERSION, XTERM_PATCH, ProgramName);
     fprintf(stderr, "where options include:\n");
-    for (opt = options; opt->opt; opt++) {
+    for (opt = list; opt->opt; opt++) {
 	fprintf(stderr, "    %-28s %s\n", opt->opt, opt->desc);
     }
 
@@ -725,14 +790,29 @@ ConvertConsoleSelection(Widget w GCC_UNUSED,
     return False;
 }
 
-Arg ourTopLevelShellArgs[] =
+#if OPT_SESSION_MGT
+static void
+die_callback(Widget w GCC_UNUSED,
+	     XtPointer client_data GCC_UNUSED,
+	     XtPointer call_data GCC_UNUSED)
 {
-    {XtNallowShellResize, (XtArgVal) TRUE},
-    {XtNinput, (XtArgVal) TRUE},
-};
-int number_ourTopLevelShellArgs = 2;
+    Cleanup(0);
+}
 
-Bool waiting_for_initial_map;
+static void
+save_callback(Widget w GCC_UNUSED,
+	      XtPointer client_data GCC_UNUSED,
+	      XtPointer call_data)
+{
+    XtCheckpointToken token = (XtCheckpointToken) call_data;
+    /* we have nothing to save */
+    token->save_success = True;
+}
+#endif /* OPT_SESSION_MGT */
+
+#if OPT_WIDE_CHARS
+int (*my_wcwidth) (wchar_t);
+#endif
 
 /*
  * DeleteWindow(): Action proc to implement ICCCM delete_window.
@@ -777,8 +857,6 @@ XtActionsRec actionProcs[] =
     {"KeyboardMapping", KeyboardMapping},
 };
 
-Atom wm_delete_window;
-
 char **gblenvp;
 extern char **environ;
 
@@ -789,11 +867,12 @@ main(int argc, char **argv ENVP_ARG)
     register TScreen *screen;
     int mode;
     char *my_class = DEFCLASS;
+    Window winToEmbedInto = None;
 
     /* Do these first, since we may not be able to open the display */
     ProgramName = argv[0];
-    TRACE_OPTS(options, optionDescList, XtNumber(optionDescList));
-    TRACE_ARGV("Before XtAppInitialize", argv);
+    TRACE_OPTS(xtermOptions, optionDescList, XtNumber(optionDescList));
+    TRACE_ARGV("Before XtOpenApplication", argv);
     if (argc > 1) {
 	int n;
 	int unique = 2;
@@ -861,19 +940,26 @@ main(int argc, char **argv ENVP_ARG)
     d_tio.c_cc[VEOL] = CEOL;	/* '^@' */
 
     /* Init the Toolkit. */
-    {
-	XtSetErrorHandler(xt_error);
-	toplevel = XtAppInitialize(&app_con, my_class,
-				   optionDescList,
-				   XtNumber(optionDescList),
-				   &argc, argv, fallback_resources,
-				   NULL, 0);
-	XtSetErrorHandler((XtErrorHandler) 0);
+    XtSetErrorHandler(xt_error);
+#if OPT_SESSION_MGT
+    toplevel = XtOpenApplication(&app_con, my_class,
+				 optionDescList,
+				 XtNumber(optionDescList),
+				 &argc, argv, fallback_resources,
+				 sessionShellWidgetClass,
+				 NULL, 0);
+#else
+    toplevel = XtAppInitialize(&app_con, my_class,
+			       optionDescList,
+			       XtNumber(optionDescList),
+			       &argc, argv, fallback_resources,
+			       NULL, 0);
+#endif /* OPT_SESSION_MGT */
+    XtSetErrorHandler((XtErrorHandler) 0);
 
-	XtGetApplicationResources(toplevel, (XtPointer) & resource,
-				  application_resources,
-				  XtNumber(application_resources), NULL, 0);
-    }
+    XtGetApplicationResources(toplevel, (XtPointer) & resource,
+			      application_resources,
+			      XtNumber(application_resources), NULL, 0);
 
     waiting_for_initial_map = resource.wait_for_map;
 
@@ -937,7 +1023,7 @@ main(int argc, char **argv ENVP_ARG)
 #endif
 
     /* Parse the rest of the command line */
-    TRACE_ARGV("After XtAppInitialize", argv);
+    TRACE_ARGV("After XtOpenApplication", argv);
     for (argc--, argv++; argc > 0; argc--, argv++) {
 	if (**argv != '-')
 	    Syntax(*argv);
@@ -975,33 +1061,33 @@ main(int argc, char **argv ENVP_ARG)
 	    continue;
 #endif /* DEBUG */
 	case 'c':		/* -class param */
-	    argc--, argv++;
+	    if (strcmp(argv[0] + 1, "class") == 0)
+		argc--, argv++;
+	    else
+		Syntax(*argv);
 	    continue;
 	case 'e':
 	    if (argc <= 1)
 		Syntax(*argv);
 	    command_to_exec = ++argv;
 	    break;
+	case 'i':
+	    if (argc <= 1) {
+		Syntax(*argv);
+	    } else {
+		char *endPtr;
+		--argc;
+		++argv;
+		winToEmbedInto = (Window) strtol(argv[0], &endPtr, 10);
+	    }
+	    break;
+
 	default:
 	    Syntax(*argv);
 	}
 	break;
     }
 
-#if OPT_WIDE_CHARS
-    /* Test whether UTF-8 mode should be active by default */
-    {
-	char *s;
-
-	if (((s = getenv("LC_ALL")) != 0 && *s != '\0') ||
-	    ((s = getenv("LC_CTYPE")) != 0 && *s != '\0') ||
-	    ((s = getenv("LANG")) != 0 && *s != '\0')) {
-	    if (strstr(s, "UTF-8"))
-		defaultUTF8[0] = '2';
-	}
-    }
-#endif
-
     SetupMenus(toplevel, &form_top, &menu_top);
 
     term = (XtermWidget) XtVaCreateManagedWidget("vt100", xtermWidgetClass,
@@ -1014,7 +1100,7 @@ main(int argc, char **argv ENVP_ARG)
 						 XtNright, XawChainRight,
 						 XtNbottom, XawChainBottom,
 #endif
-						 0);
+						 (XtPointer) 0);
     /* this causes the initialize method to be called */
 
 #if OPT_HP_FUNC_KEYS
@@ -1039,10 +1125,23 @@ main(int argc, char **argv ENVP_ARG)
 	inhibit |= I_TEK;
 #endif
 
-/*
- * Set title and icon name if not specified
- */
+#if OPT_WIDE_CHARS
+    my_wcwidth = &mk_wcwidth;
+    if (term->misc.cjk_width)
+	my_wcwidth = &mk_wcwidth_cjk;
+#endif
 
+#if OPT_SESSION_MGT
+    if (resource.sessionMgt) {
+	TRACE(("Enabling session-management callbacks\n"));
+	XtAddCallback(toplevel, XtNdieCallback, die_callback, NULL);
+	XtAddCallback(toplevel, XtNsaveCallback, save_callback, NULL);
+    }
+#endif
+
+    /*
+     * Set title and icon name if not specified
+     */
     if (command_to_exec) {
 	Arg args[2];
 
@@ -1064,6 +1163,37 @@ main(int argc, char **argv ENVP_ARG)
 
 	XtSetValues(toplevel, args, 2);
     }
+#if OPT_LUIT_PROG
+    if (term->misc.callfilter) {
+	int u = (term->misc.use_encoding ? 2 : 0);
+	if (command_to_exec) {
+	    int n;
+	    char **c;
+	    for (n = 0, c = command_to_exec; *c; n++, c++) ;
+	    c = malloc((n + 3 + u) * sizeof(char *));
+	    if (c == NULL)
+		SysError(ERROR_LUMALLOC);
+	    memcpy(c + 2 + u, command_to_exec, (n + 1) * sizeof(char *));
+	    c[0] = term->misc.localefilter;
+	    if (u) {
+		c[1] = "-encoding";
+		c[2] = term->misc.locale_str;
+	    }
+	    c[1 + u] = "--";
+	    command_to_exec_with_luit = c;
+	} else {
+	    static char *luit[4];
+	    luit[0] = term->misc.localefilter;
+	    if (u) {
+		luit[1] = "-encoding";
+		luit[2] = term->misc.locale_str;
+		luit[3] = NULL;
+	    } else
+		luit[1] = NULL;
+	    command_to_exec_with_luit = luit;
+	}
+    }
+#endif
 #if OPT_TEK4014
     if (inhibit & I_TEK)
 	screen->TekEmu = FALSE;
@@ -1080,7 +1210,7 @@ main(int argc, char **argv ENVP_ARG)
 	int i = -1;
 	if (debug) {
 	    creat_as(getuid(), getgid(), True, "xterm.debug.log", 0666);
-	    i = open("xterm.debug.log", O_WRONLY | O_TRUNC, 0666);
+	    i = open("xterm.debug.log", O_WRONLY | O_TRUNC);
 	}
 	if (i >= 0) {
 	    dup2(i, 2);
@@ -1112,11 +1242,11 @@ main(int argc, char **argv ENVP_ARG)
     screen->inhibit = inhibit;
 
     if (0 > (mode = fcntl(screen->respond, F_GETFL, 0)))
-	Error(1);
+	SysError(ERROR_F_GETFL);
     mode |= O_NDELAY;
 
     if (fcntl(screen->respond, F_SETFL, mode))
-	Error(1);
+	SysError(ERROR_F_SETFL);
 
     FD_ZERO(&pty_mask);
     FD_ZERO(&X_mask);
@@ -1141,6 +1271,19 @@ main(int argc, char **argv ENVP_ARG)
 	StartLog(screen);
     }
 #endif
+
+    if (winToEmbedInto != None) {
+	XtRealizeWidget(toplevel);
+	/*
+	 * This should probably query the tree or check the attributes of
+	 * winToEmbedInto in order to verify that it exists, but I'm still not
+	 * certain what is the best way to do it -GPS
+	 */
+	XReparentWindow(XtDisplay(toplevel),
+			XtWindow(toplevel),
+			winToEmbedInto, 0, 0);
+    }
+
     for (;;) {
 #if OPT_TEK4014
 	if (screen->TekEmu)
@@ -1279,6 +1422,19 @@ first_map_occurred(void)
     waiting_for_initial_map = False;
 }
 
+static void
+set_owner(char *device, int uid, int gid, int mode)
+{
+    if (chown(device, uid, gid) < 0) {
+	if (errno != ENOENT
+	    && getuid() == 0) {
+	    fprintf(stderr, "Cannot chown %s to %d,%d: %s\n",
+		    device, uid, gid, strerror(errno));
+	}
+    }
+    chmod(device, mode);
+}
+
 #define THE_PARENT 1
 #define THE_CHILD  2
 int whoami = -1;
@@ -1327,7 +1483,7 @@ spawn(void)
     char *ptr, *shname, buf[64];
     int i, no_dev_tty = FALSE, envsize;
     char *dev_tty_name = (char *) 0;
-    struct winsize ws;
+    TTYSIZE_STRUCT ts;
     int pgrp = getpid();
     char numbuf[12], **envnew;
 
@@ -1355,7 +1511,7 @@ spawn(void)
 	signal(SIGALRM, hungtty);
 	alarm(2);		/* alarm(1) might return too soon */
 	if (!setjmp(env)) {
-	    tty = open("/dev/tty", O_RDWR, 0);
+	    tty = open("/dev/tty", O_RDWR);
 	    alarm(0);
 	    tty_got_hung = False;
 	} else {
@@ -1380,6 +1536,7 @@ spawn(void)
 		SysError(ERROR_OPDEVTTY);
 	    }
 	} else {
+
 	    /* Get a copy of the current terminal's state,
 	     * if we can.  Some systems (e.g., SVR4 and MacII)
 	     * may not have a controlling terminal at this point
@@ -1438,19 +1595,16 @@ spawn(void)
      * the program to proceed (but not to set $TERMCAP) if the termcap
      * entry is not found.
      */
-    get_termcap(TermName = resource.term_name, ptr, newtc);
-
-    /*
-     * This block is invoked only if there was no terminal name specified
-     * by the command-line option "-tn".
-     */
-    if (!TermName) {
+    if (!get_termcap(TermName = resource.term_name, ptr, newtc)) {
+	char *last = NULL;
 	TermName = *envnew;
 	while (*envnew != NULL) {
-	    if (get_termcap(*envnew, ptr, newtc)) {
+	    if ((last == NULL || strcmp(last, *envnew))
+		&& get_termcap(*envnew, ptr, newtc)) {
 		TermName = *envnew;
 		break;
 	    }
+	    last = *envnew;
 	    envnew++;
 	}
     }
@@ -1458,17 +1612,17 @@ spawn(void)
     /* tell tty how big window is */
 #if OPT_TEK4014
     if (TEK4014_ACTIVE(screen)) {
-	ws.ws_row = 38;
-	ws.ws_col = 81;
-	ws.ws_xpixel = TFullWidth(screen);
-	ws.ws_ypixel = TFullHeight(screen);
+	TTYSIZE_ROWS(ts) = 38;
+	TTYSIZE_COLS(ts) = 81;
+	ts.ws_xpixel = TFullWidth(screen);
+	ts.ws_ypixel = TFullHeight(screen);
     } else
 #endif
     {
-	ws.ws_row = screen->max_row + 1;
-	ws.ws_col = screen->max_col + 1;
-	ws.ws_xpixel = FullWidth(screen);
-	ws.ws_ypixel = FullHeight(screen);
+	TTYSIZE_ROWS(ts) = screen->max_row + 1;
+	TTYSIZE_COLS(ts) = screen->max_col + 1;
+	ts.ws_xpixel = FullWidth(screen);
+	ts.ws_ypixel = FullHeight(screen);
     }
 
     if (am_slave < 0) {
@@ -1502,7 +1656,7 @@ opencons();*/
 	    /* Now is the time to set up our process group and
 	     * open up the pty slave.
 	     */
-	    if ((tty = open(ttydev, O_RDWR, 0)) < 0) {
+	    if ((tty = open(ttydev, O_RDWR)) < 0) {
 		/* dumm gelaufen */
 		fprintf(stderr, "Cannot open slave side of PTY\n");
 		exit(1);
@@ -1514,7 +1668,11 @@ opencons();*/
 #ifdef EMXNOTBOGUS
 	    if ((ptr = ttyname(tty)) != 0) {
 		/* it may be bigger */
-		ttydev = realloc(ttydev, (unsigned) (strlen(ptr) + 1));
+		ttydev = realloc(ttydev,
+				 (unsigned) (strlen(ptr) + 1));
+		if (ttydev == NULL) {
+		    SysError(ERROR_SPREALLOC);
+		}
 		(void) strcpy(ttydev, ptr);
 	    }
 #else
@@ -1524,10 +1682,8 @@ opencons();*/
 	    ptioctl(tty, XTY_ENADUP, 0);
 
 	    /* change ownership of tty to real group and user id */
-	    chown(ttydev, screen->uid, screen->gid);
-
-	    /* change protection of tty */
-	    chmod(ttydev, (resource.messages ? 0622 : 0600));
+	    set_owner(ttydev, screen->uid, screen->gid,
+		      (resource.messages ? 0622 : 0600));
 
 	    /* for the xf86sup-pty, we set the pty to bypass: OS/2 does
 	     * not have a line discipline structure
@@ -1604,10 +1760,10 @@ opencons();*/
 	    if (handshake.rows > 0 && handshake.cols > 0) {
 		screen->max_row = handshake.rows;
 		screen->max_col = handshake.cols;
-		ws.ws_row = screen->max_row + 1;
-		ws.ws_col = screen->max_col + 1;
-		ws.ws_xpixel = FullWidth(screen);
-		ws.ws_ypixel = FullHeight(screen);
+		TTYSIZE_ROWS(ts) = screen->max_row + 1;
+		TTYSIZE_COLS(ts) = screen->max_col + 1;
+		ts.ws_xpixel = FullWidth(screen);
+		ts.ws_ypixel = FullHeight(screen);
 	    }
 
 	    sprintf(numbuf, "%d", screen->max_col + 1);
@@ -1619,7 +1775,7 @@ opencons();*/
 	    environ = gblenvp;
 
 	    /* need to reset after all the ioctl bashing we did above */
-	    ptioctl(0, TIOCSWINSZ, (char *) &ws);
+	    ptioctl(0, TIOCSWINSZ, (char *) &ts);
 
 	    signal(SIGHUP, SIG_DFL);
 
@@ -1633,6 +1789,22 @@ opencons();*/
 		DosCloseEventSem(sev);
 	    }
 
+#if OPT_LUIT_PROG
+	    /*
+	     * Use two copies of command_to_exec, in case luit is not actually
+	     * there, or refuses to run.  In that case we will fall-through to
+	     * to command that the user gave anyway.
+	     */
+	    if (command_to_exec_with_luit) {
+		TRACE(("spawning command \"%s\"\n", *command_to_exec_with_luit));
+		execvp(*command_to_exec_with_luit, command_to_exec_with_luit);
+		/* print error message on screen */
+		fprintf(stderr, "%s: Can't execvp %s: %s\n",
+			xterm_name, *command_to_exec_with_luit, strerror(errno));
+		fprintf(stderr, "%s: cannot support your locale.\n",
+			xterm_name);
+	    }
+#endif
 	    if (command_to_exec) {
 		TRACE(("spawning command \"%s\"\n", *command_to_exec));
 		execvpe(*command_to_exec, command_to_exec, gblenvp);
@@ -1718,12 +1890,8 @@ Exit(int n)
 #endif
     if (am_slave < 0) {
 	/* restore ownership of tty and pty */
-	chown(ttydev, 0, 0);
-	chown(ptydev, 0, 0);
-
-	/* restore modes of tty and pty */
-	chmod(ttydev, 0666);
-	chmod(ptydev, 0666);
+	set_owner(ttydev, 0, 0, 0666);
+	set_owner(ptydev, 0, 0, 0666);
     }
     exit(n);
     SIGNAL_RETURN;
@@ -1731,7 +1899,7 @@ Exit(int n)
 
 /* ARGSUSED */
 static void
-resize(TScreen * screen, register char *oldtc, register char *newtc)
+resize(TScreen * screen, register char *oldtc, char *newtc)
 {
 }
 
@@ -1753,6 +1921,7 @@ nonblocking_wait(void)
 static SIGNAL_T
 reapchild(int n GCC_UNUSED)
 {
+    int olderrno = errno;
     int pid;
 
     pid = wait(NULL);
@@ -1773,6 +1942,7 @@ reapchild(int n GCC_UNUSED)
 	}
     } while ((pid = nonblocking_wait()) > 0);
 
+    errno = olderrno;
     SIGNAL_RETURN;
 }
 
@@ -1791,6 +1961,7 @@ parse_tty_modes(char *s, struct _xttymodes *modelist)
     int c;
     int count = 0;
 
+    TRACE(("parse_tty_modes\n"));
     while (1) {
 	while (*s && isascii(CharOf(*s)) && isspace(CharOf(*s)))
 	    s++;
@@ -1810,25 +1981,12 @@ parse_tty_modes(char *s, struct _xttymodes *modelist)
 	if (!*s)
 	    return -1;
 
-	if (*s == '^') {
-	    s++;
-	    c = ((*s == '?') ? 0177 : CONTROL(*s));
-	    if (*s == '-') {
-		errno = 0;
-		c = fpathconf(0, _PC_VDISABLE);
-		if (c == -1) {
-		    if (errno != 0)
-			continue;	/* skip this (error) */
-		    c = 0377;
-		}
-	    }
-	} else {
-	    c = *s;
+	if ((c = decode_keyvalue(&s, False)) != -1) {
+	    mp->value = c;
+	    mp->set = 1;
+	    count++;
+	    TRACE(("...parsed #%d: %s=%#x\n", count, mp->name, c));
 	}
-	mp->value = c;
-	mp->set = 1;
-	count++;
-	s++;
     }
 }
 
@@ -1900,12 +2058,12 @@ ptioctl(int fd, int func, void *data)
 			   NULL, 0, NULL);
     case TIOCSWINSZ:
 	return DosDevIOCtl(fd, XFREE86_PTY, XTY_TIOCSWINSZ,
-			   (ULONG *) data, sizeof(struct winsize), &len,
+			   (ULONG *) data, sizeof(TTYSIZE_STRUCT), &len,
 			   NULL, 0, NULL);
     case TIOCGWINSZ:
 	return DosDevIOCtl(fd, XFREE86_PTY, XTY_TIOCGWINSZ,
 			   NULL, 0, NULL,
-			   (ULONG *) data, sizeof(struct winsize), &len);
+			   (ULONG *) data, sizeof(TTYSIZE_STRUCT), &len);
     case XTY_ENADUP:
 	i = 1;
 	return DosDevIOCtl(fd, XFREE86_PTY, XTY_ENADUP,
