@@ -2,10 +2,10 @@
  *	$Xorg: ptyx.h,v 1.3 2000/08/17 19:55:09 cpqbld Exp $
  */
 
-/* $XFree86: xc/programs/xterm/ptyx.h,v 3.94 2002/06/01 00:54:49 dickey Exp $ */
+/* $XFree86: xc/programs/xterm/ptyx.h,v 3.103 2003/03/23 17:18:55 dickey Exp $ */
 
 /*
- * Copyright 1999,2000,2001,2002 by Thomas E. Dickey
+ * Copyright 1999-2002,2003 by Thomas E. Dickey
  *
  *                         All Rights Reserved
  *
@@ -67,6 +67,8 @@
 /* @(#)ptyx.h	X10/6.6	11/10/86 */
 
 #include <X11/IntrinsicP.h>
+#include <X11/Shell.h>		/* for XtNdieCallback, etc. */
+#include <X11/StringDefs.h>	/* for standard resource names */
 #include <X11/Xmu/Misc.h>	/* For Max() and Min(). */
 #include <X11/Xfuncs.h>
 #include <X11/Xosdefs.h>
@@ -84,17 +86,6 @@
 #define MyStackFree(pointer, stack_cache_array) \
     if ((pointer) != ((char *)(stack_cache_array))) free(pointer)
 
-#ifdef AMOEBA
-/* Avoid name clashes with standard Amoeba types: */
-#define event    am_event_t
-#define interval am_interval_t
-#include <amoeba.h>
-#include <semaphore.h>
-#include <circbuf.h>
-#undef event
-#undef interval
-#endif
-
 /*
 ** System V definitions
 */
@@ -124,7 +115,7 @@
 #define USE_PTY_DEVICE 1
 #define USE_PTY_SEARCH 1
 
-#if defined(__osf__) || (defined(linux) && defined(__GLIBC__) && (__GLIBC__ >= 2) && (__GLIBC_MINOR__ >= 1))
+#if defined(__osf__) || (defined(linux) && defined(__GLIBC__) && (__GLIBC__ >= 2) && (__GLIBC_MINOR__ >= 1)) || defined(__NetBSD__)
 #undef USE_PTY_DEVICE
 #undef USE_PTY_SEARCH
 #define USE_PTS_DEVICE 1
@@ -137,14 +128,14 @@
 
 #if defined(SYSV) && defined(i386) && !defined(SVR4)
 #define ATT
-#define USE_HANDSHAKE
+#define USE_HANDSHAKE 1
 #define USE_ISPTS_FLAG 1
 #endif
 
 #if (defined(ATT) && !defined(__sgi)) || defined(__MVS__) || (defined(SYSV) && defined(i386)) || (defined (__GLIBC__) && ((__GLIBC__ > 2) || (__GLIBC__ == 2) && (__GLIBC_MINOR__ >= 1)))
 #define USE_USG_PTYS
 #else
-#define USE_HANDSHAKE
+#define USE_HANDSHAKE 1
 #endif
 
 /*
@@ -389,7 +380,11 @@ typedef struct {
 
 #define Cres(name,class,offset,value) \
 	{RES_NAME(name), RES_CLASS(class), XtRPixel, sizeof(Pixel), \
-	 RES_OFFSET(offset), XtRString, value}
+	 RES_OFFSET(offset), XtRString, (XtPointer) value}
+
+#define Fres(name,class,offset,value) \
+	{RES_NAME(name), RES_CLASS(class), XtRFontStruct, sizeof(XFontStruct *), \
+	 RES_OFFSET(offset), XtRString, (XtPointer) value}
 
 #define Ires(name,class,offset,value) \
 	{RES_NAME(name), RES_CLASS(class), XtRInt, sizeof(int), \
@@ -419,6 +414,10 @@ typedef struct {
 #define OPT_BOX_CHARS	1 /* true if xterm can simulate box-characters */
 #endif
 
+#ifndef OPT_C1_PRINT
+#define OPT_C1_PRINT	1 /* true if xterm allows C1 controls to be printable */
+#endif
+
 #ifndef OPT_CLIP_BOLD
 #define OPT_CLIP_BOLD	1 /* true if xterm uses clipping to avoid bold-trash */
 #endif
@@ -491,30 +490,57 @@ typedef struct {
 #define OPT_HIGHLIGHT_COLOR 1 /* true if xterm supports color highlighting */
 #endif
 
+#ifndef OPT_LUIT_PROG
+#define OPT_LUIT_PROG   0 /* true if xterm supports luit */
+#endif
+
 #ifndef OPT_MAXIMIZE
 #define OPT_MAXIMIZE	1 /* add actions for iconify ... maximize */
 #endif
 
+#ifndef OPT_MOD_FKEYS
+#define OPT_MOD_FKEYS	1 /* modify cursor- and function-keys in normal mode */
+#endif
+
 #ifndef OPT_NUM_LOCK
 #define OPT_NUM_LOCK	1 /* use NumLock key only for numeric-keypad */
 #endif
 
-#ifndef OPT_SAME_NAME
-#define OPT_SAME_NAME   1 /* suppress redundant updates of title, icon, etc. */
-#endif
-
 #ifndef OPT_PC_COLORS
 #define OPT_PC_COLORS   1 /* true if xterm supports PC-style (bold) colors */
 #endif
 
+#ifndef OPT_PTY_HANDSHAKE
+#ifndef USE_HANDSHAKE
+#define USE_HANDSHAKE	0
+#endif
+#define OPT_PTY_HANDSHAKE USE_HANDSHAKE	/* avoid pty races on older systems */
+#endif
+
 #ifndef OPT_PRINT_COLORS
 #define OPT_PRINT_COLORS 1 /* true if we print color information */
 #endif
 
+#ifndef OPT_READLINE
+#define OPT_READLINE	0 /* mouse-click/paste support for readline */
+#endif
+
+#ifndef OPT_SAME_NAME
+#define OPT_SAME_NAME   1 /* suppress redundant updates of title, icon, etc. */
+#endif
+
 #ifndef OPT_SCO_FUNC_KEYS
 #define OPT_SCO_FUNC_KEYS 0 /* true if xterm supports SCO-style function keys */
 #endif
 
+#ifndef OPT_SESSION_MGT
+#if defined(XtNdieCallback) && defined(XtNsaveCallback)
+#define OPT_SESSION_MGT 1
+#else
+#define OPT_SESSION_MGT 0
+#endif
+#endif
+
 #ifndef OPT_SHIFT_FONTS
 #define OPT_SHIFT_FONTS 1 /* true if xterm interprets fontsize-shifting */
 #endif
@@ -659,7 +685,7 @@ typedef struct {
 
 /* Define a fake XK code, we need it for the fake color response in
  * xtermcapKeycode(). */
-#ifdef OPT_TCAP_QUERY
+#if OPT_TCAP_QUERY
 # define XK_COLORS 0x0003
 #endif
 
@@ -738,6 +764,10 @@ typedef struct {
 #define CurMaxCol(screen, row) screen->max_col
 #define CurCursorX(screen, row, col) CursorX(screen, col)
 #define CurFontWidth(screen, row) FontWidth(screen)
+#endif
+
+#if OPT_LUIT_PROG && !OPT_WIDE_CHARS
+#error Luit requires the wide-chars configuration
 #endif
 
 	/* the number of pointers per row in 'ScrnBuf' */
@@ -945,6 +975,7 @@ typedef enum {
 
 	/* index into vt_shell[] or tek_shell[] */
 typedef enum {
+	noMenu = -1,
 	mainMenu,
 	vtMenu,
 	fontMenu,
@@ -1040,11 +1071,6 @@ typedef struct {
 					   (position report, etc.)	*/
 #if OPT_TCAP_QUERY
 	int		tc_query;
-#endif
-#ifdef AMOEBA
-	capability      proccap;        /* process capability           */
-	struct circbuf  *tty_inq;       /* tty server input queue       */
-	struct circbuf  *tty_outq;      /* tty server output queue      */
 #endif
 	long		pid;		/* pid of process on far side   */
 	int		uid;		/* user id of actual person	*/
@@ -1061,7 +1087,6 @@ typedef struct {
 	int		han_ascent;
 	int		han_descent;
 	XPoint		*han_box;
-	String		menu_bfont_names[NMENUFONTS];
 	int		han_code; /* Hangul code 1|2|3 */
 	int		han_kbd; /* 2 Ȥ 3  */
 	int		han_status_height;
@@ -1101,12 +1126,23 @@ typedef struct {
 	int		utf8_mode;	/* use UTF-8 decode/encode: 0-2	*/
 	int		utf_count;	/* state of utf_char */
 	IChar		utf_char;	/* in-progress character */
+#endif
+#if OPT_C1_PRINT
+	Boolean		c1_printable;	/* true if we treat C1 as print	*/
 #endif
 	int		border;		/* inner border			*/
 	Cursor		arrow;		/* arrow cursor			*/
 	unsigned long	event_mask;
 	unsigned short	send_mouse_pos;	/* user wants mouse transition  */
 					/* and position information	*/
+#if OPT_READLINE
+	unsigned	click1_moves;
+	unsigned	paste_moves;
+	unsigned	dclick3_deletes;
+	unsigned	paste_brackets;
+	unsigned	paste_quotes;
+	unsigned	paste_literal_nl;
+#endif	/* OPT_READLINE */
 #if OPT_DEC_LOCATOR
 	Boolean		locator_reset;	/* turn mouse off after 1 report? */
 	Boolean		locator_pixels;	/* report in pixels?		*/
@@ -1125,6 +1161,7 @@ typedef struct {
 	Boolean		visualbell;	/* visual bell mode		*/
 	Boolean		poponbell;	/* pop on bell mode		*/
 	Boolean		allowSendEvents;/* SendEvent mode		*/
+	Boolean		allowWindowOps;	/* WindowOps mode		*/
 	Boolean		awaitInput;	/* select-timeout mode		*/
 	Boolean		grabbedKbd;	/* keyboard is grabbed		*/
 #ifdef ALLOWLOGGING
@@ -1301,7 +1338,7 @@ typedef struct {
 	Boolean		trim_selection; /* controls trimming of selection */
 	Boolean		i18nSelections;
 	Boolean		brokenSelections;
-	char		*selection_data; /* the current selection */
+	Char		*selection_data; /* the current selection */
 	int		selection_size; /* size of allocated buffer */
 	int		selection_length; /* number of significant bytes */
 	Time		selection_time;	/* latest event timestamp */
@@ -1318,7 +1355,7 @@ typedef struct {
 	Boolean		meta_sends_esc;		/* Meta-key sends ESC prefix */
 	Pixmap		menu_item_bitmap;	/* mask for checking items */
 	String		menu_font_names[NMENUFONTS];
-	unsigned long	menu_font_sizes[NMENUFONTS];
+	long		menu_font_sizes[NMENUFONTS];
 	int		menu_font_number;
 	XIC		xic;
 #ifdef XRENDERFONT
@@ -1326,7 +1363,7 @@ typedef struct {
 	XftFont		*renderFontBold;
 	XftDraw		*renderDraw;
 #endif
-#ifdef OPT_INPUT_METHOD
+#if OPT_INPUT_METHOD
 	XFontSet	fs;		/* fontset for XIM preedit */
 	int		fs_ascent;	/* ascent of fs */
 #endif
@@ -1343,7 +1380,16 @@ typedef struct _TekPart {
 #endif
 } TekPart;
 
-
+#if OPT_READLINE
+#define SCREEN_FLAG(screenp,f)		(1&(screenp)->f)
+#define SCREEN_FLAG_set(screenp,f)	((screenp)->f |= 1)
+#define SCREEN_FLAG_unset(screenp,f)	((screenp)->f &= ~1L)
+#define SCREEN_FLAG_save(screenp,f)	\
+	((screenp)->f = (((screenp)->f)<<1) | SCREEN_FLAG(screenp,f))
+#define SCREEN_FLAG_restore(screenp,f)	((screenp)->f = (((screenp)->f)>>1))
+#else
+#define SCREEN_FLAG(screenp,f)		(0)
+#endif
 
 /* meaning of bits in screen.select flag */
 #define	INWINDOW	01	/* the mouse is in one of the windows */
@@ -1367,6 +1413,7 @@ typedef struct
 #if OPT_INITIAL_ERASE
     int	reset_DECBKM;		/* reset should set DECBKM */
 #endif
+    int modify_cursor_keys;	/* how to handle modifiers */
 } TKeyboard;
 
 typedef struct _Misc {
@@ -1377,6 +1424,13 @@ typedef struct _Misc {
 #if OPT_WIDE_CHARS
     char *f_w;
     char *f_wb;
+    Boolean	cjk_width;	/* true when CJK width convention is turned on */
+#endif
+#if OPT_LUIT_PROG
+    Boolean callfilter;		/* true to invoke luit */
+    Boolean use_encoding;	/* true to use -encoding option for luit */
+    char *locale_str;		/* "locale" resource */
+    char *localefilter;		/* path for luit */
 #endif
 #if OPT_INPUT_METHOD
     char *f_x;
@@ -1443,6 +1497,7 @@ typedef struct _Misc {
     Boolean	han_keep_baseline;
     Boolean	han_no_status_kbd;
     Boolean	han_no_status_code;
+    Boolean	han_allow_shuffle;
 #if defined(OPT_NOW_CHAT_SCROLL_PATCH)
     Boolean	han_now_chat;
 #endif
