@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright 1999-2000 by Thomas E. Dickey
+ * Copyright 1999-2002,2003 by Thomas E. Dickey
  *
  *                         All Rights Reserved
  *
@@ -54,7 +54,7 @@
  * SOFTWARE.
  */
 
-/* $XFree86: xc/programs/xterm/screen.c,v 3.59 2002/04/28 19:04:21 dickey Exp $ */
+/* $XFree86: xc/programs/xterm/screen.c,v 3.61 2003/03/23 02:01:40 dickey Exp $ */
 
 /* screen.c */
 
@@ -116,13 +116,12 @@ Allocate(int nrow, int ncol, Char ** addr)
  *  (Return value only necessary with SouthWestGravity.)
  */
 static int
-Reallocate(
-	      ScrnBuf * sbuf,
-	      Char ** sbufaddr,
-	      int nrow,
-	      int ncol,
-	      int oldrow,
-	      int oldcol)
+Reallocate(ScrnBuf * sbuf,
+	   Char ** sbufaddr,
+	   int nrow,
+	   int ncol,
+	   int oldrow,
+	   int oldcol)
 {
     ScrnBuf base;
     Char *tmp;
@@ -218,12 +217,11 @@ int last_written_col = -1;
  * to match flags.
  */
 void
-ScreenWrite(
-	       TScreen * screen,
-	       PAIRED_CHARS(Char * str, Char * str2),
-	       unsigned flags,
-	       unsigned cur_fg_bg,
-	       int len)		/* length of string */
+ScreenWrite(TScreen * screen,
+	    PAIRED_CHARS(Char * str, Char * str2),
+	    unsigned flags,
+	    unsigned cur_fg_bg,
+	    int len)		/* length of string */
 {
 #if OPT_ISO_COLORS
 #if OPT_EXT_COLORS
@@ -701,14 +699,12 @@ ScrnDeleteChar(TScreen * screen, int n)
  *	     all dimensions are based on single-characters.
  */
 void
-ScrnRefresh(
-	       TScreen * screen,
-	       int toprow,
-	       int leftcol,
-	       int nrows,
-	       int ncols,
-	       Bool force
-)				/* ... leading/trailing spaces */
+ScrnRefresh(TScreen * screen,
+	    int toprow,
+	    int leftcol,
+	    int nrows,
+	    int ncols,
+	    Bool force)		/* ... leading/trailing spaces */
 {
     int y = toprow * FontHeight(screen) + screen->border;
     int row;
@@ -781,6 +777,7 @@ ScrnRefresh(
 	else
 	    lastind = row - scrollamt;
 
+	TRACE(("ScrnRefresh row=%d lastind=%d/%d\n", row, lastind, max));
 	if (lastind < 0 || lastind > max)
 	    continue;
 
@@ -941,8 +938,9 @@ ScrnRefresh(
 		|| (hilite && (col > hi_col))
 #if OPT_ISO_COLORS
 		|| ((flags & FG_COLOR)
-		    && ((extract_fg(ColorOf(col), attrs[col]) != fg)
-			|| (extract_bg(ColorOf(col), attrs[col]) != bg)))
+		    && (extract_fg(ColorOf(col), attrs[col]) != fg))
+		|| ((flags & BG_COLOR)
+		    && (extract_bg(ColorOf(col), attrs[col]) != bg))
 #endif
 #if OPT_WIDE_CHARS
 		|| (widec
@@ -955,10 +953,9 @@ ScrnRefresh(
 		) {
 		TRACE(("ScrnRefresh looping drawXtermText %d..%d:%s\n",
 		       lastind, col,
-		       visibleChars(
-				       PAIRED_CHARS(&chars[lastind],
-						    WIDEC_PTR(lastind)),
-				       col - lastind)));
+		       visibleChars(PAIRED_CHARS(&chars[lastind],
+						 WIDEC_PTR(lastind)),
+				    col - lastind)));
 
 		p = &chars[lastind];
 		test = flags;
@@ -1132,14 +1129,14 @@ ScrnRefresh(
 
 #if defined(__CYGWIN__) && defined(TIOCSWINSZ)
     if (first_time == 1) {
-	struct winsize ws;
+	TTYSIZE_STRUCT ts;
 
 	first_time = 0;
-	ws.ws_row = nrows;
-	ws.ws_col = ncols;
-	ws.ws_xpixel = term->core.width;
-	ws.ws_ypixel = term->core.height;
-	ioctl(screen->respond, TIOCSWINSZ, (char *) &ws);
+	TTYSIZE_ROWS(ts) = nrows;
+	TTYSIZE_COLS(ts) = ncols;
+	ts.ws_xpixel = term->core.width;
+	ts.ws_ypixel = term->core.height;
+	SET_TTYSIZE(screen->respond, ts);
     }
 #endif
     recurse--;
@@ -1150,10 +1147,9 @@ ScrnRefresh(
  * Requires first <= last; first, last are rows of screen->buf.
  */
 void
-ClearBufRows(
-		TScreen * screen,
-		int first,
-		int last)
+ClearBufRows(TScreen * screen,
+	     int first,
+	     int last)
 {
     ScrnBuf buf = screen->visbuf;
     int len = screen->max_col + 1;
@@ -1202,21 +1198,18 @@ ClearBufRows(
   8. Returns 0
   */
 int
-ScreenResize(
-		TScreen * screen,
-		int width,
-		int height,
-		unsigned *flags)
+ScreenResize(TScreen * screen,
+	     int width,
+	     int height,
+	     unsigned *flags)
 {
     int code;
     int rows, cols;
     int border = 2 * screen->border;
     int move_down_by;
-#if defined(TIOCSSIZE) && (defined(sun) && !defined(SVR4))
-    struct ttysize ts;
-#elif defined(TIOCSWINSZ)
-    struct winsize ws;
-#endif /* sun vs TIOCSWINSZ */
+#ifdef TTYSIZE_STRUCT
+    TTYSIZE_STRUCT ts;
+#endif
     Window tw = VWindow(screen);
 
     TRACE(("ScreenResize %dx%d\n", height, width));
@@ -1343,13 +1336,18 @@ ScreenResize(
     }
 #endif /* NO_ACTIVE_ICON */
 
-#if defined(TIOCSSIZE) && (defined(sun) && !defined(SVR4))
+#ifdef TTYSIZE_STRUCT
     /* Set tty's idea of window size */
-    ts.ts_lines = rows;
-    ts.ts_cols = cols;
-    code = ioctl(screen->respond, TIOCSSIZE, &ts);
-    TRACE(("return %d from TIOCSSIZE %dx%d\n", code, rows, cols));
-#ifdef SIGWINCH
+    TTYSIZE_ROWS(ts) = rows;
+    TTYSIZE_COLS(ts) = cols;
+#ifdef USE_STRUCT_WINSIZE
+    ts.ws_xpixel = width;
+    ts.ws_ypixel = height;
+#endif
+    code = SET_TTYSIZE(screen->respond, ts);
+    TRACE(("return %d from SET_TTYSIZE %dx%d\n", code, rows, cols));
+
+#if defined(SIGWINCH) && defined(USE_STRUCT_TTYSIZE)
     if (screen->pid > 1) {
 	int pgrp;
 
@@ -1357,25 +1355,10 @@ ScreenResize(
 	    kill_process_group(pgrp, SIGWINCH);
     }
 #endif /* SIGWINCH */
-#elif defined(TIOCSWINSZ)
-    /* Set tty's idea of window size */
-    ws.ws_row = rows;
-    ws.ws_col = cols;
-    ws.ws_xpixel = width;
-    ws.ws_ypixel = height;
-    code = ioctl(screen->respond, TIOCSWINSZ, (char *) &ws);
-    TRACE(("return %d from TIOCSWINSZ %dx%d\n", code, rows, cols));
-#ifdef notdef			/* change to SIGWINCH if this doesn't work for you */
-    if (screen->pid > 1) {
-	int pgrp;
 
-	if (ioctl(screen->respond, TIOCGPGRP, &pgrp) != -1)
-	    kill_process_group(pgrp, SIGWINCH);
-    }
-#endif /* SIGWINCH */
 #else
     TRACE(("ScreenResize cannot do anything to pty\n"));
-#endif /* sun vs TIOCSWINSZ */
+#endif /* TTYSIZE_STRUCT */
     return (0);
 }
 
@@ -1384,11 +1367,10 @@ ScreenResize(
  * nonnull.
  */
 Bool
-non_blank_line(
-		  ScrnBuf sb,
-		  int row,
-		  int col,
-		  int len)
+non_blank_line(ScrnBuf sb,
+	       int row,
+	       int col,
+	       int len)
 {
     int i;
     Char *ptr = BUF_CHARS(sb, row);
