@@ -2,7 +2,7 @@
  *	$Xorg: scrollbar.c,v 1.4 2000/08/17 19:55:09 cpqbld Exp $
  */
 
-/* $XFree86: xc/programs/xterm/scrollbar.c,v 3.33 2002/04/28 19:04:21 dickey Exp $ */
+/* $XFree86: xc/programs/xterm/scrollbar.c,v 3.36 2002/12/27 21:05:23 dickey Exp $ */
 
 /*
  * Copyright 2000-2001,2002 by Thomas E. Dickey
@@ -59,8 +59,6 @@
 #include <xterm.h>
 
 #include <X11/Xatom.h>
-#include <X11/StringDefs.h>
-#include <X11/Shell.h>
 
 #if defined(HAVE_LIB_XAW)
 #include <X11/Xaw/Scrollbar.h>
@@ -183,7 +181,7 @@ ResizeScreen(XtermWidget xw, int min_width, int min_height)
 		  XtNheightInc, FontHeight(screen),
 		  XtNminWidth, min_width + FontWidth(screen),
 		  XtNminHeight, min_height + FontHeight(screen),
-		  NULL);
+		  (XtPointer) 0);
 
     reqWidth = screen->fullVwin.f_width * (screen->max_col + 1) + min_width;
 #if OPT_HANGUL
@@ -409,7 +407,7 @@ ScrollBarOn(XtermWidget xw, int init, int doalloc)
 						      screen->savelines) *
 			       sizeof(char *)))
 	    == NULL)
-	      Error(ERROR_SBRALLOC);
+	      SysError(ERROR_SBRALLOC);
 	screen->visbuf = &screen->allbuf[MAX_PTRS * screen->savelines];
 	memmove((char *) screen->visbuf, (char *) screen->allbuf,
 		MAX_PTRS * (screen->max_row + 2) * sizeof(char *));
@@ -420,7 +418,7 @@ ScrollBarOn(XtermWidget xw, int init, int doalloc)
 		     (Char *) calloc((unsigned) screen->max_col + 1,
 				     sizeof(Char))
 		    ) == NULL)
-		    Error(ERROR_SBRALLOC2);
+		    SysError(ERROR_SBRALLOC2);
 	    }
 	}
     }
@@ -448,7 +446,7 @@ ScrollBarOn(XtermWidget xw, int init, int doalloc)
 	&& screen->fullVwin.fullwidth < term->core.width)
 	XtVaSetValues(screen->scrollWidget,
 		      XtNx, screen->fullVwin.fullwidth - screen->scrollWidget->core.border_width,
-		      NULL);
+		      (XtPointer) 0);
 #endif
 
     XtMapWidget(screen->scrollWidget);
