@@ -2,10 +2,10 @@
  *	$Xorg: util.c,v 1.3 2000/08/17 19:55:10 cpqbld Exp $
  */
 
-/* $XFree86: xc/programs/xterm/util.c,v 3.71 2002/04/28 19:04:21 dickey Exp $ */
+/* $XFree86: xc/programs/xterm/util.c,v 3.74 2003/03/09 23:39:14 dickey Exp $ */
 
 /*
- * Copyright 1999-2001,2002 by Thomas E. Dickey
+ * Copyright 1999-2002,2003 by Thomas E. Dickey
  *
  *                         All Rights Reserved
  *
@@ -72,8 +72,6 @@
 #include <stdio.h>
 #include <ctype.h>
 
-extern Bool waiting_for_initial_map;
-
 static int ClearInLine(TScreen * screen, int row, int col, int len);
 static int handle_translated_exposure(TScreen * screen,
 				      int rect_x,
@@ -1502,7 +1500,7 @@ drawXtermText(TScreen * screen,
 	    screen->renderDraw = XftDrawCreate(dpy, draw, visual,
 					       DefaultColormap(dpy, scr));
 	}
-	if ((flags & (BOLD | BLINK)) && screen->renderFontBold)
+	if ((flags & (BOLD | BLINK)) && !screen->colorBDMode && screen->renderFontBold)
 	    font = screen->renderFontBold;
 	else
 	    font = screen->renderFont;
@@ -1531,7 +1529,41 @@ drawXtermText(TScreen * screen,
 			    x, y, sbuf, len);
 	} else
 #endif
-	{
+	if (!screen->force_box_chars) {
+	    /* adding code to substitute simulated line-drawing characters */
+	    Cardinal last, first = 0;
+	    Dimension old_wide, old_high = 0;
+
+	    for (last = 0; last < len; last++) {
+		unsigned ch = text[last];
+		if (ch > 0 && ch < 32) {
+		    /* line drawing character time */
+		    if (last > first) {
+			XftDrawString8(screen->renderDraw,
+				       getColor(values.foreground),
+				       font,
+				       x + (first * FontWidth(screen)), y,
+				       (unsigned char *) text + first,
+				       last - first);
+		    }
+		    old_wide = screen->fnt_wide;
+		    old_high = screen->fnt_high;
+		    screen->fnt_wide = FontWidth(screen);
+		    screen->fnt_high = FontHeight(screen);
+		    xtermDrawBoxChar(screen, ch, flags, gc,
+				     x + (last * FontWidth(screen)),
+				     y - FontAscent(screen));
+		    screen->fnt_wide = old_wide;
+		    screen->fnt_high = old_high;
+		    first = last + 1;
+		}
+	    }
+	    if (last > first) {
+		XftDrawString8(screen->renderDraw, getColor(values.foreground),
+			       font, x + (first * FontWidth(screen)), y,
+			       (unsigned char *) text + first, last - first);
+	    }
+	} else {
 	    XftDrawString8(screen->renderDraw,
 			   getColor(values.foreground),
 			   font,
@@ -1742,7 +1774,7 @@ drawXtermText(TScreen * screen,
 
     /* If the font is complete, draw it as-is */
     if (screen->fnt_boxes && !screen->force_box_chars) {
-	TRACE(("drawtext%c[%4d,%4d] (%d) %d:%s\n",
+	TRACE(("drawXtermText%c[%4d,%4d] (%d) %d:%s\n",
 	       screen->cursor_state == OFF ? ' ' : '*',
 	       y, x, chrset, len,
 	       visibleChars(PAIRED_CHARS(text, text2), len)));
@@ -2141,10 +2173,10 @@ putXtermCell(TScreen * screen, int row, int col, int ch)
 }
 
 /*
- * Add a the combining character for the given cell
+ * Add a combining character for the given cell
  */
 void
-addXtermCombining(TScreen * screen, int row, int col, int ch)
+addXtermCombining(TScreen * screen, int row, int col, unsigned ch)
 {
     if (!SCRN_BUF_COM1L(screen, row)[col]
 	&& !SCRN_BUF_COM1H(screen, row)[col]) {
