@@ -1,8 +1,8 @@
-/* $XFree86: xc/programs/xterm/xterm.h,v 3.80 2002/06/01 00:54:50 dickey Exp $ */
+/* $XFree86: xc/programs/xterm/xterm.h,v 3.88 2003/03/09 23:39:14 dickey Exp $ */
 
 /************************************************************
 
-Copyright 1999,2000,2001,2002 by Thomas E. Dickey
+Copyright 1999-2002,2003 by Thomas E. Dickey
 
                         All Rights Reserved
 
@@ -46,7 +46,11 @@ authorization.
 #endif
 
 #ifndef GCC_UNUSED
-#define GCC_UNUSED /* nothing */
+#define GCC_UNUSED		/* nothing */
+#endif
+
+#ifndef GCC_NORETURN
+#define GCC_NORETURN		/* nothing */
 #endif
 
 #include <X11/Xos.h>
@@ -86,7 +90,7 @@ authorization.
 #endif
 
 #if defined(hpux) && !defined(__hpux)
-#define __hpux 1	/* HPUX 11.0 does not define this */
+#define __hpux 1		/* HPUX 11.0 does not define this */
 #endif
 
 #ifdef USE_POSIX_TERMIOS
@@ -94,7 +98,7 @@ authorization.
 #define HAVE_TCGETATTR 1
 #endif
 
-#if defined(__UNIXOS2__) || defined(__CYGWIN__) || defined(SCO) || defined(sco)
+#if defined(__UNIXOS2__) || defined(SCO) || defined(sco)
 #define USE_TERMCAP 1
 #endif
 
@@ -129,7 +133,7 @@ authorization.
 #define USE_LASTLOG
 #endif
 
-#if defined(AMOEBA) || defined(SCO) || defined(SVR4) || defined(_POSIX_SOURCE) || defined(__QNX__) || defined(__hpux) || (defined(BSD) && (BSD >= 199103)) || defined(__CYGWIN__)
+#if defined(SCO) || defined(SVR4) || defined(_POSIX_SOURCE) || defined(__QNX__) || defined(__hpux) || (defined(BSD) && (BSD >= 199103)) || defined(__CYGWIN__)
 #define USE_POSIX_WAIT
 #endif
 
@@ -160,6 +164,10 @@ authorization.
 #define HAVE_UTMP_UT_XSTATUS 1
 #endif
 
+#if defined(XKB)
+#define HAVE_XKBBELL 1
+#endif
+
 #endif /* HAVE_CONFIG_H */
 
 /***====================================================================***/
@@ -250,7 +258,7 @@ extern int errno;
 #include <sys/types.h>
 
 #if defined(USE_XPOLL_H) && defined(AIXV3) && defined(NFDBITS)
-#undef NFDBITS	/* conflict between X11/Xpoll.h and sys/select.h */
+#undef NFDBITS			/* conflict between X11/Xpoll.h and sys/select.h */
 #endif
 
 #include <sys/select.h>
@@ -261,7 +269,9 @@ extern int errno;
 
 /***====================================================================***/
 
+#define XtNallowC1Printable	"allowC1Printable"
 #define XtNallowSendEvents	"allowSendEvents"
+#define XtNallowWindowOps	"allowWindowOps"
 #define XtNalwaysHighlight	"alwaysHighlight"
 #define XtNalwaysUseMods	"alwaysUseMods"
 #define XtNanswerbackString	"answerbackString"
@@ -278,6 +288,7 @@ extern int errno;
 #define XtNc132			"c132"
 #define XtNcacheDoublesize	"cacheDoublesize"
 #define XtNcharClass		"charClass"
+#define XtNcjkWidth		"cjkWidth"
 #define XtNcolorAttrMode	"colorAttrMode"
 #define XtNcolorBDMode		"colorBDMode"
 #define XtNcolorBLMode		"colorBLMode"
@@ -312,6 +323,8 @@ extern int errno;
 #define XtNjumpScroll		"jumpScroll"
 #define XtNkeyboardDialect	"keyboardDialect"
 #define XtNlimitResize		"limitResize"
+#define XtNlocale		"locale"
+#define XtNlocaleFilter		"localeFilter"
 #define XtNlogFile		"logFile"
 #define XtNlogInhibit		"logInhibit"
 #define XtNlogging		"logging"
@@ -320,6 +333,7 @@ extern int errno;
 #define XtNmenuBar		"menuBar"
 #define XtNmenuHeight		"menuHeight"
 #define XtNmetaSendsEscape	"metaSendsEscape"
+#define XtNmodifyCursorKeys	"modifyCursorKeys"
 #define XtNmultiClickTime	"multiClickTime"
 #define XtNmultiScroll		"multiScroll"
 #define XtNnMarginBell		"nMarginBell"
@@ -368,7 +382,9 @@ extern int errno;
 #define XtNxmcInline		"xmcInline"
 #define XtNxmcMoveSGR		"xmcMoveSGR"
 
+#define XtCAllowC1Printable	"AllowC1Printable"
 #define XtCAllowSendEvents	"AllowSendEvents"
+#define XtCAllowWindowOps	"AllowWindowOps"
 #define XtCAlwaysHighlight	"AlwaysHighlight"
 #define XtCAlwaysUseMods	"AlwaysUseMods"
 #define XtCAnswerbackString	"AnswerbackString"
@@ -384,6 +400,8 @@ extern int errno;
 #define XtCC132			"C132"
 #define XtCCacheDoublesize	"CacheDoublesize"
 #define XtCCharClass		"CharClass"
+#define XtCCjkWidth 		"CjkWidth"
+#define XtCColorAttrMode        "ColorAttrMode"
 #define XtCColorMode		"ColorMode"
 #define XtCColumn		"Column"
 #define XtCCtrlFKeys		"CtrlFKeys"
@@ -409,6 +427,8 @@ extern int errno;
 #define XtCJumpScroll		"JumpScroll"
 #define XtCKeyboardDialect	"KeyboardDialect"
 #define XtCLimitResize		"LimitResize"
+#define XtCLocale		"Locale"
+#define XtCLocaleFilter		"LocaleFilter"
 #define XtCLogInhibit		"LogInhibit"
 #define XtCLogfile		"Logfile"
 #define XtCLogging		"Logging"
@@ -417,6 +437,7 @@ extern int errno;
 #define XtCMenuBar		"MenuBar"
 #define XtCMenuHeight		"MenuHeight"
 #define XtCMetaSendsEscape	"MetaSendsEscape"
+#define XtCModifyCursorKeys	"ModifyCursorKeys"
 #define XtCMultiClickTime	"MultiClickTime"
 #define XtCMultiScroll		"MultiScroll"
 #define XtCNumLock		"NumLock"
@@ -446,8 +467,9 @@ extern int errno;
 #define XtCTrimSelection	"TrimSelection"
 #define XtCUnderLine		"UnderLine"
 #define XtCUtf8			"Utf8"
-#define XtCVisualBell		"VisualBell"
+#define XtCVeryBoldColors	"VeryBoldColors"
 #define XtCVT100Graphics	"VT100Graphics"
+#define XtCVisualBell		"VisualBell"
 #define XtCWideBoldFont		"WideBoldFont"
 #define XtCXimFont		"XimFont"
 #define XtCWideChars		"WideChars"
@@ -458,7 +480,7 @@ extern int errno;
 #define XtCXmcInline		"XmcInline"
 #define XtCXmcMoveSGR		"XmcMoveSGR"
 
-#ifdef NO_ACTIVE_ICON
+#if defined(NO_ACTIVE_ICON) && !defined(XtNgeometry)
 #define XtNgeometry		"geometry"
 #define XtCGeometry		"Geometry"
 #endif
@@ -475,7 +497,7 @@ extern int errno;
 
 /***====================================================================***/
 
-#ifdef	__cplusplus
+#ifdef __cplusplus
 extern "C" {
 #endif
 
@@ -509,6 +531,7 @@ extern void HandleSelectExtend        PROTO_XT_ACTIONS_ARGS;
 extern void HandleSelectSet           PROTO_XT_ACTIONS_ARGS;
 extern void HandleSelectStart         PROTO_XT_ACTIONS_ARGS;
 extern void HandleStartExtend         PROTO_XT_ACTIONS_ARGS;
+extern void ReadLineButton            PROTO_XT_ACTIONS_ARGS;
 extern void ResizeSelection (TScreen *screen, int rows, int cols);
 extern void ScrollSelection (TScreen* screen, int amount);
 extern void TrackMouse (int func, int startrow, int startcol, int firstrow, int lastrow);
@@ -615,12 +638,19 @@ extern void first_map_occurred (void);
 extern SIGNAL_T Exit (int n);
 #endif
 
+#if OPT_WIDE_CHARS
+extern int (*my_wcwidth)(wchar_t);
+#endif
+
 /* menu.c */
 extern void do_hangup          PROTO_XT_CALLBACK_ARGS;
 extern void show_8bit_control  (Bool value);
 
 /* misc.c */
 extern Cursor make_colored_cursor (unsigned cursorindex, unsigned long fg, unsigned long bg);
+extern OptionHelp * sortedOpts(OptionHelp *, XrmOptionDescRec *, Cardinal);
+extern Window WMFrameWindow(XtermWidget termw);
+extern XrmOptionDescRec * sortedOptDescs(XrmOptionDescRec *, Cardinal);
 extern char *SysErrorMsg (int n);
 extern char *udk_lookup (int keycode, int *len);
 extern int XStrCmp (char *s1, char *s2);
@@ -633,7 +663,6 @@ extern void ChangeXprop (char *name);
 extern void Changename (char *name);
 extern void Changetitle (char *name);
 extern void Cleanup (int code);
-extern void Error (int i);
 extern void HandleBellPropertyChange PROTO_XT_EV_HANDLER_ARGS;
 extern void HandleEightBitKeyPressed PROTO_XT_ACTIONS_ARGS;
 extern void HandleEnterWindow        PROTO_XT_EV_HANDLER_ARGS;
@@ -645,7 +674,7 @@ extern void HandleStringEvent        PROTO_XT_ACTIONS_ARGS;
 extern void Panic (char *s, int a);
 extern void Redraw (void);
 extern void ReverseOldColors (void);
-extern void SysError (int i);
+extern void SysError (int i) GCC_NORETURN;
 extern void VisualBell (void);
 extern void do_dcs (Char *buf, size_t len);
 extern void do_osc (Char *buf, int len, int final);
@@ -681,7 +710,9 @@ extern void FlushLog (TScreen *screen);
 #endif
 
 /* print.c */
+extern Boolean xtermHasPrinter (void);
 extern int xtermPrinterControl (int chr);
+extern void setPrinterControlMode (int mode);
 extern void xtermAutoPrint (int chr);
 extern void xtermMediaControl (int param, int private_seq);
 extern void xtermPrintScreen (Boolean use_DECPEX);
@@ -863,7 +894,7 @@ extern Pixel xtermGetColorRes(ColorRes *res);
 extern unsigned getXtermCell (TScreen *screen, int row, int col);
 extern unsigned getXtermCellComb1 (TScreen *screen, int row, int col);
 extern unsigned getXtermCellComb2 (TScreen *screen, int row, int col);
-extern void addXtermCombining (TScreen *screen, int row, int col, int ch);
+extern void addXtermCombining (TScreen *screen, int row, int col, unsigned ch);
 extern void putXtermCell (TScreen *screen, int row, int col, int ch);
 #else
 #define getXtermCell(screen,row,col) SCRN_BUF_CHARS(screen, row)[col]
@@ -882,7 +913,7 @@ int visual_width(PAIRED_CHARS(Char *str, Char *str2), Cardinal len);
 #define visual_width(a, b) (b)
 #endif
 
-#ifdef	__cplusplus
+#ifdef __cplusplus
 	}
 #endif
 
